/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTDialog.jsm");
XPCOMUtils.importRelative (this, "UnMHTPrototype.jsm");
XPCOMUtils.importRelative (this, "UnMHTSave.jsm");
XPCOMUtils.importRelative (this, "UnMHTTimer.jsm");

let EXPORTED_SYMBOLS = ["UnMHTAdditionalDialogClass"];

/**
 * 追加ファイルダイアログ
 *
 * ウィンドウの引数
 * {
 *   callback : Function  終了時のコールバック (必須)
 * }
 *
 * @param  nsIDOMWindow window
 *         対象のウィンドウ
 * @inherits   UnMHTDialog
 * @implements IUnMHTDialog
 */
function UnMHTAdditionalDialogClass (window) {
  UnMHTDialog.call (this, window);
}
UnMHTAdditionalDialogClass.prototype = {
  /* ==== IUnMHTDialog インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct : function () {
    this._callback = null;
    UnMHTPrototype.super (this, "destruct").call (this);
  },
  
  /**
   * ロードイベント
   */
  onLoad : function () {
    let arg = this._window.arguments [0];
    
    if ("callback" in arg) {
      this._callback = arg.callback;
    }
  },
  
  /**
   * ダイアログを OK したイベント
   *
   * @return Boolean
   *         ダイアログを閉じるか
   */
  onDialogAccept : function () {
    if (this._callback) {
      let addURIs = UnMHTSave.parseAddURIs (this.el ("add_uris").value);
      let callback = this._callback;
      
      UnMHTTimer.setTimeout
      (function () {
        callback (addURIs);
      }, 100);
    }
        
    return true;
  },
  
  /* ==== private ==== */
  
  _callback : null, /* Function  終了時コールバック */
};
UnMHTPrototype.createInheritance (UnMHTAdditionalDialogClass, UnMHTDialog);
