/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;

let EXPORTED_SYMBOLS = ["UnMHTArrayUtil"];

/**
 * 配列ユーティリティ
 */
let UnMHTArrayUtil = {
  /**
   * 配列 a の中で最初に cond を満たした要素を返す
   * どの要素も cond を満たさない場合は fail を返す
   *
   * 速度が必要な場面で filter の代わりに使う
   * 下の 2 つは等価
   * [x] = a.filter (f);
   * x = UnMHTArrayUtil.find (a, f, undefined);
   *
   * @param  Array a
   *         対象の配列
   * @param  Function cond
   *         条件
   * @param  Any fail
   *         失敗時の返り値
   * @return Any
   *         a の要素もしくは fail
   */
  find : function (a, cond, fail) {
    for (let i = 0; i < a.length; i ++) {
      if (cond (a [i], i, a)) {
        return a [i];
      }
    }
    
    return fail;
  },
  
  /**
   * Array.reduce の途中結果も含めて配列で返す
   * Haskell の Data.List の scanl
   *
   * @param  Array a
   *         対象の配列
   * @param  Function func
   *         適用する関数   
   * @param  Any init
   *         初期値
   * @return Array
   *         結果の配列
   */
  scanl : function (a, func, init) {
    return a.map (function (item) {
        return (init = func (init, item));
      });
  },
  
  /**
   * [begin, end) のイテレータを返す
   *
   * @param  Number begin
   *         始値
   * @param  Number end
   *         終値
   * @return Iterator
   *         [begin, end) のイテレータ
   */
  range : function (begin, end) {
    if (begin < end) {
      for (let i = begin; i < end; i ++) {
        yield i;
      }
    }
    else {
      for (let i = begin; i > end; i --) {
        yield i;
      }
    }
  }
};
