/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;
const Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTExtractor.jsm");
XPCOMUtils.importRelative (this, "UnMHTStringBundle.jsm");
XPCOMUtils.importRelative (this, "UnMHTHTMLUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative (this, "UnMHTFileUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative (this, "UnMHTUnicodeConverter.jsm");

let EXPORTED_SYMBOLS = ["UnMHTCachePage"];

/**
 * キャッシュページ
 */
let UnMHTCachePage = {
  /**
   * キャッシュページを作成する
   *
   * @return String
   *         キャッシュページのソース
   */
  createCachePage : function () {
    let tableRow = function (header, data) {
      return ("<tr>"
              + "<th>" + header + "</th>"
              + "<td>" + data + "</td>"
              + "</tr>");
    };
    let bundle = function (name) {
      return UnMHTHTMLUtil.escapeEntity (UnMHTStringBundle.get (name));
    };
    
    let size = 0;
    
    let eFileInfoList = UnMHTExtractor.getFileInfoList ();
    
    let bodies
    = eFileInfoList.map (function (eFileInfo) {
        let body = [];
        
        size += eFileInfo.size;
        
        let param = UnMHTScheme.getUnMHTURIParam (eFileInfo.baseURI);
        let original = param.original;
        if (param.scheme == "file") {
          original = UnMHTFileUtil.getFileFromURLSpec (original).path;
        }
        
        body.push ("<table border=\"0\">");
        
        body.push (tableRow ("URI: ",
                             "<a href=\""
                             + UnMHTHTMLUtil.escapeEntity (eFileInfo.baseURI)
                             + "\">"
                             + UnMHTHTMLUtil.escapeEntity (eFileInfo.baseURI)
                             + "</a>"));
        body.push (tableRow (bundle ("original_address") + " ",
                             UnMHTHTMLUtil.escapeEntity (original)));
        body.push (tableRow (bundle ("opened_time") + " ",
                             UnMHTHTMLUtil.escapeEntity
                             (UnMHTFormatter.getReadableTime
                              (eFileInfo.extractTime))));
        body.push (tableRow (bundle ("last_access_time") + " ",
                             UnMHTHTMLUtil.escapeEntity
                             (UnMHTFormatter.getReadableTime
                              (eFileInfo.lastAccessTime))));
        body.push (tableRow (bundle ("file_count") + " ",
                             eFileInfo.parts.length));
        
        if (eFileInfo.date) {
          let savedTime = eFileInfo.date;
          try {
            savedTime = UnMHTFormatter.getReadableTime
              ((new Date (savedTime)).getTime ());
          }
          catch (e) {
          }
          
          body.push (tableRow (bundle ("saved_time") + " ",
                               UnMHTHTMLUtil.escapeEntity (savedTime)));
        }
        
        if (eFileInfo.subject) {
          body.push (tableRow ("Subject: ",
                               UnMHTHTMLUtil.escapeEntity (eFileInfo.subject)));
        }
        
        body.push ("</table>");
        
        eFileInfo.parts.forEach (function (part) {
            body.push ("<table border=\"0\" class=\"file\">");
            body.push (tableRow (bundle ("ref_name") + " ",
                                 "<a href=\""
                                 + UnMHTHTMLUtil.escapeEntity
                                 (eFileInfo.baseURI)
                                 + UnMHTHTMLUtil.escapeEntity
                                 (part.eParam.refName)
                                 + "\">"
                                 + UnMHTHTMLUtil.escapeEntity
                                 (part.eParam.refName)
                                 + "</a>"
                                 + ((eFileInfo.startPart == part)
                                    ? (" (" + bundle ("start_part") + ")")
                                    : "")));
            if (part.contentLocation) {
              body.push (tableRow ("Content-Location: ",
                                   UnMHTHTMLUtil.escapeEntity
                                   (part.contentLocation)));
            }
            if (part.eParam.contentBase) {
              body.push (tableRow ("Content-Base: ",
                                   UnMHTHTMLUtil.escapeEntity
                                   (part.eParam.contentBase)));
            }
            if (part.contentID) {
              body.push (tableRow ("Content-ID: ",
                                   UnMHTHTMLUtil.escapeEntity
                                   (part.contentID)));
            }
            if (part.contentDispositionType
                && part.contentDispositionFilename) {
              body.push (tableRow ("Content-Disposition: ",
                                   UnMHTHTMLUtil.escapeEntity
                                   (part.contentDispositionType)
                                   + "; filename="
                                   + UnMHTHTMLUtil.escapeEntity
                                   (part.contentDispositionFilename)));
            }
            body.push (tableRow ("Content-Type: ",
                                 UnMHTHTMLUtil.escapeEntity
                                 (part.eParam.mimetype)
                                 + (part.eParam.charset
                                    ? ("; charset=" + UnMHTHTMLUtil.escapeEntity
                                       (part.eParam.charset))
                                    : "")));
            if (part.contentTransferEncoding) {
              body.push (tableRow ("Content-Encoding: ",
                                   UnMHTHTMLUtil.escapeEntity
                                   (part.contentTransferEncoding)));
            }
            body.push (tableRow (bundle ("file_size") + " ",
                                 UnMHTFormatter.getReadableSize
                                 (part.eParam.content.length)
                                 + " " + bundle ("bytes")));
            body.push ("</table>");
          });
        
        return body.join ("");
      })
    
    let data
    = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    + "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
    + "          \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">"
    + "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
    + "<head>"
    + "<title>" + bundle ("cache_title") + "</title>"
    + "<style type=\"text/css\">"
    + "th {"
    + "text-align: right;"
    + "white-space: nowrap;"
    + "}"
    + "table {"
    + "margin-bottom: 1em;"
    + "}"
    + "table.file {"
    + "margin-bottom: 1em;"
    + "margin-left: 1em;"
    + "}"
    + "</style>"
    + "</head>"
    + "<body>"
    + "<h1>" + bundle ("cache_title") + "</h1>"
    
    + "<hr />"
    + "<table border=\"0\">"
        
    + tableRow (bundle ("cache_entry") + " ",
                eFileInfoList.length)
        
    + tableRow (bundle ("cache_size") + " ",
                UnMHTFormatter.getReadableSize (size) + " " + bundle ("bytes"))
    
    + "</table>"
    + "<hr />"
    + bodies.join ("<hr />");
    + "</body>"
    + "</html>";
    
    data = UnMHTUnicodeConverter.convertFromUnicode (data, "utf-8");
    
    return data;
  }
};
