/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
let Cm = Components.manager;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTShutdown.jsm");
XPCOMUtils.importRelative (this, "UnMHTThreadUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTComponentLoader"];

/**
 * XPCOM のロードとアンロード
 */
let UnMHTComponentLoader = {
  /**
   * XPCOM を登録してアンロード時のリスナを登録も行う
   *
   * @param  nsISupport component
   *         登録するコンポーネント
   */
  loadComponent : function (component) {
    let factory = {
      createInstance: function (outer, iid) {
        if (outer) {
          throw Cr.NS_ERROR_NO_AGGREGATION;
        }
        return (new component ()).QueryInterface (iid);
      }
    };
  
    this._registerComponent (component, factory);
  
    UnMHTShutdown.addListener (function (data, reason) {
        this._unregisterComponent (component, factory);
      }.bind (this));
  },
  
  /* ==== private ==== */
  
  /**
   * XPCOM を登録する
   *
   * @param  nsISupport component
   *         登録するコンポーネント
   * @param  nsIFactory factory
   *         登録するコンポーネントの Factory
   */
  _registerComponent : function (component, factory) {
    let manager = Cm.QueryInterface (Ci.nsIComponentRegistrar);
    
    manager.registerFactory
    (component.prototype.classID,
     component.prototype.classDescription,
     component.prototype.contractID,
     factory);
    
    if ("_xpcom_categories" in component.prototype) {
      component.prototype._xpcom_categories.forEach (function (cat) {
          XPCOMUtils.categoryManager.addCategoryEntry
            (cat.category,
             component.prototype.contractID,
             component.prototype.contractID,
             false, true);
        });
    }
  },

  /**
   * XPCOM の登録を解除する
   *
   * @param  nsISupport component
   *         解除するコンポーネント
   * @param  nsIFactory factory
   *         解除するコンポーネントの Factory
   */
  _unregisterComponent : function (component, factory) {
    if ("_xpcom_categories" in component.prototype) {
      component.prototype._xpcom_categories.forEach (function (cat) {
          XPCOMUtils.categoryManager.deleteCategoryEntry
          (cat.category,
           component.prototype.contractID,
           false);
        });
    }
    
    let manager = Cm.QueryInterface (Ci.nsIComponentRegistrar);
    UnMHTThreadUtil.runAsync (function () {
        manager.unregisterFactory
          (component.prototype.classID, factory);
      }.bind (this))
  }
};
