/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/Services.jsm");
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTFileUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTPref.jsm");
XPCOMUtils.importRelative (this, "UnMHTStringBundle.jsm");

let EXPORTED_SYMBOLS = ["UnMHTCustomFilename"];

/**
 * カスタムファイル名フォーマッタ
 */
let UnMHTCustomFilename = {
  /**
   * カスタムファイル名をフォーマットする
   * 設定は自動取得する
   *
   * @param  String TITTLE
   *         ページのタイトル
   * @param  String FILENAME
   *         ページのファイル名
   * @param  String URL
   *         ページの URL
   * @param  Date date
   *         日時
   * @return String
   *         フォーマットされた文字列
   */
  format : function (TITLE, FILENAME, URL, date) {
    let pattern = UnMHTPref.getPref ("save.filename.custom");
    let am = UnMHTPref.getPref ("save.filename.custom.am");
    let pm = UnMHTPref.getPref ("save.filename.custom.pm");
    
    return this.formatWith (TITLE, FILENAME, URL, date, pattern, am, pm);
  },

  /**
   * カスタムファイル名をフォーマットする
   *
   * @param  String TITTLE
   *         ページのタイトル
   * @param  String FILENAME
   *         ページのファイル名
   * @param  String URL
   *         ページの URL
   * @param  Date date
   *         日時
   * @param  String pattern
   *         カスタムファイル名フォーマット文字列
   * @param  String am
   *         a.m. の文字列
   * @param  String pm
   *         p.m. の文字列
   * @return String
   *         フォーマットされた文字列
   */
  formatWith : function (TITLE, FILENAME, URL, date, pattern, am, pm) {
    let uri = Services.io.newURI (URL, null, null);
    
    let pad2 = function (n) {
      return ("0" + n).slice (-2);
    };
    
    let port = uri.port;
    if (port == -1) {
      if (uri.scheme == "http") {
        port = 80;
      }
      else if (uri.scheme == "https") {
        port = 443;
      }
      else if (uri.scheme == "ftp") {
        port = 21;
      }
    }
    
    let [, path, query, fragment]
    = uri.path.match (/([^\?#]*)(?:\?([^#]*))?(?:#(.*))?/);
    query = query || "";
    fragment = fragment || "";
    
    if (UnMHTPref.getPref ("save.filename.custom.limit")) {
      let limitCount
        = UnMHTPref.getPref ("save.filename.custom.limit.count");
      TITLE = TITLE.substr (0, limitCount);
      FILENAME = FILENAME.substr (0, limitCount);
    }
    
    let vars = {
      "YYYY" : date.getFullYear (),
      "Y"    : date.getFullYear () % 100,
      "YY"   : pad2 (date.getFullYear () % 100),
      "M"    : date.getMonth () + 1,
      "MM"   : pad2 (date.getMonth () + 1),
      "MMM"  : this._weekdays [date.getDay ()],
      "MMMM" : this._weekdays2 [date.getDay ()],
      "D"    : date.getDate (),
      "DD"   : pad2 (date.getDate ()),
      "H"    : date.getHours (),
      "HH"   : pad2 (date.getHours ()),
      "h"    : date.getHours () % 12,
      "hh"   : pad2 (date.getHours () % 12),
      "m"    : date.getMinutes (),
      "mm"   : pad2 (date.getMinutes ()),
      "s"    : date.getSeconds (),
      "ss"   : pad2 (date.getSeconds ()),
      "ampm" : date.getHours () >= 12 ? pm : am,
      
      "TITLE"    : TITLE,
      "FILENAME" : FILENAME,
      "URL"      : URL,
      "URL2"     : URL.replace (/^[A-Za-z0-9]+:\/\//, ""),
      "SCHEME"   : uri.scheme,
      "HOST"     : uri.host,
      "PORT"     : port,
      "FULLPATH" : uri.path,
      "PATH"     : path,
      "QUERY"    : query,
      "FRAGMENT" : fragment
    };
    
    let rexp = new RegExp ("%(" + Object.keys (vars).join ("|") + ")%", "g");
    
    let text = pattern.replace (rexp, function (matched, name) vars [name]);
    
    return UnMHTFileUtil.validateFileName (text);
  },
  
  /* ==== private ==== */

  /* Array  曜日 (省略) */
  _weekdays : [
    UnMHTStringBundle.get ("weekday_sun"),
    UnMHTStringBundle.get ("weekday_mon"),
    UnMHTStringBundle.get ("weekday_tue"),
    UnMHTStringBundle.get ("weekday_wed"),
    UnMHTStringBundle.get ("weekday_thu"),
    UnMHTStringBundle.get ("weekday_fri"),
    UnMHTStringBundle.get ("weekday_sat")
    ],
  
  /* Array  曜日 */
  _weekdays2 : [
    UnMHTStringBundle.get ("weekday2_sun"),
    UnMHTStringBundle.get ("weekday2_mon"),
    UnMHTStringBundle.get ("weekday2_tue"),
    UnMHTStringBundle.get ("weekday2_wed"),
    UnMHTStringBundle.get ("weekday2_thu"),
    UnMHTStringBundle.get ("weekday2_fri"),
    UnMHTStringBundle.get ("weekday2_sat")
    ]
};
