/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTExtractor.jsm");
XPCOMUtils.importRelative (this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative (this, "UnMHTPrototype.jsm");
XPCOMUtils.importRelative (this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowHook.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTInfoPanelClass"];

/**
 * 情報パネル
 *
 * @param  nsIDOMWindow window
 *         対象のウィンドウ
 * @inherits   UnMHTWindowHook
 * @implements nsISupportsWeakReference
 * @implements nsIWebProgressListener
 * @implements IUnMHTWindowHook
 */
function UnMHTInfoPanelClass (window) {
  UnMHTWindowHook.call (this, window);
  
  this.onInfoIconClick = this.onInfoIconClick.bind (this);
}
UnMHTInfoPanelClass.prototype = {
  QueryInterface : XPCOMUtils.generateQI ([Ci.nsISupportsWeakReference,
                                           Ci.nsIWebProgressListener]),
  
  /**
   * 情報アイコンがクリックされたイベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onInfoIconClick : function (event) {
    let targetDocument = UnMHTWindowUtil.getCurrentDocument (this._window);
    
    let location = targetDocument.location.href;
    
    let [eFileInfo, part] = UnMHTExtractor.getFileInfoAndPart (location);
    if (!(eFileInfo && part)) {
      this._icon.hidden = true;
      return;
    }
    
    this._currentLocation = eFileInfo.original;
    this._currentSavedFrom = "";
    this._linkable = false;
    
    this._panel = this.el ("unmht-info-panel");
    
    this._panel.hidden = false;
    this.el ("unmht-info-panel").style.width
    = this.el ("urlbar").boxObject.width + "px";
    this.el ("unmht-info-panel-original").value = this._currentLocation;
    
    if (eFileInfo.startPart
        && eFileInfo.startPart.contentLocation) {
      this._currentSavedFrom
        = part.contentLocation
        || eFileInfo.startPart.contentLocation;
      
      this.el ("unmht-info-panel-row-saved-from").hidden = false;
      this.el ("unmht-info-panel-saved-from").value = this._currentSavedFrom;
      
      
      if (this._currentSavedFrom.match (/^[A-Za-z0-9\-]+:/)) {
        this._linkable = true;
      }
      else {
        this._linkable = false;
      }
      
      if (this._linkable) {
        this.el ("unmht-info-panel-saved-from").className = "text-link";
      }
      else {
        this.el ("unmht-info-panel-saved-from").className = "";
      }
    }
    else {
      this.el ("unmht-info-panel-row-saved-from").hidden = true;
    }
    if (eFileInfo.date) {
      let savedTime = eFileInfo.date;
      try {
        savedTime = UnMHTFormatter.getReadableTime
          ((new Date (savedTime)).getTime ());
      }
      catch (e) {
      }
          
      this.el ("unmht-info-panel-row-saved-date").hidden = false;
      this.el ("unmht-info-panel-saved-date").value = savedTime;
    }
    else {
      this.el ("unmht-info-panel-row-saved-date").hidden = true;
    }
    
    this._panel.openPopup (this._icon, "bottomcenter topright");
    
    event.stopPropagation ();
  },
  
  /**
   * 情報パネルのコピーボタンがクリックされたイベント
   */
  onInfoCopyClick : function () {
    Cc ["@mozilla.org/widget/clipboardhelper;1"]
    .getService (Ci.nsIClipboardHelper)
    .copyString (this._currentLocation);
  },
  
  /**
   * 情報パネルのコピーボタンがクリックされたイベント
   */
  onInfoSavedFromClick : function () {
    if (this._linkable) {
      this._panel.hidePopup ();
      let tabbrowser = UnMHTWindowUtil.getBrowser (this._window);
      tabbrowser.selectedTab = tabbrowser.addTab (this._currentSavedFrom);
    }
  },
  
  /* ==== nsIWebProgressListener インターフェース ==== */
  
  /**
   * 監視ウィンドウのロケーションが変わった時のイベント
   *   nsIWebProgressListener.onLocationChange
   *
   * @param  nsIWebProgress webProgress
   *         呼び出し元
   * @param  nsIRequest request
   *         状況の変わったリクエスト
   * @param  nsIURI location
   *         ロケーション
   */
  onLocationChange : function (webProgress, request, location) {
    this._setInfoIconState ();
  },
  
  /**
   * 進行状況が変わった時のイベント
   * 未使用
   */
  onProgressChange: function () {
  },
  
  /**
   * プロトコルのセキュリティ設定が変わった時のイベント
   * 未使用
   */
  onSecurityChange : function () {
  },
  
  /**
   * 状況が変わった時のイベント
   *
   * @param  nsIWebProgress webProgress
   *         呼び出し元
   * @param  nsIRequest request
   *         状況の変わったリクエスト
   * @param  Number flags
   *         状況フラグ
   *           Ci.nsIWebProgressListener.STATE_*
   * @param  nsresult status
   *         エラーコード
   */
  onStateChange : function (webProgress, request, flags, status) {
    if (flags & Ci.nsIWebProgressListener.STATE_STOP) {
      this._setInfoIconState ();
    }
  },
  
  /**
   * ステータスバーに表示するメッセージが変わった時のイベント
   * 未使用
   */
  onStatusChange : function () {
  },
  
  /* ==== IUnMHTWindowHook インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct : function () {
    UnMHTPrototype.super (this, "destruct").call (this);
  },
  
  /**
   * ウィンドウにロードする
   *
   * @param  Boolean isStartup
   *         スタートアップイベントか
   * @param  Number reason
   *         スタートアップの理由
   */
  load : function (isStartup, reason) {
    this._window.UnMHTInfoPanel = this;
    
    let target = UnMHTWindowUtil.getBrowser (this._window);
    if (target) {
      target.addProgressListener (this);
    }
    
    this._setInfoIconState ();
  },
  
  /**
   * ウィンドウからアンロードする
   *
   * @param  Boolean isShutdown
   *         シャットダウンイベントか
   * @param  Number reason
   *         シャットダウンの理由
   */
  unload : function (isShutdown, reason) {
    if (this._panel) {
      try {
        this._panel.hidePopup ();
      }
      catch (e) {
      }
      this._panel = null;
    }
    
    if (this._icon && this._icon.parentNode) {
      this._icon.removeEventListener
      ("click", this.onInfoIconClick, false);
      this._icon.parentNode.removeChild (this._icon);
      this._icon = null;
    }
    
    let target = UnMHTWindowUtil.getBrowser (this._window);
    if (target) {
      target.removeProgressListener (this);
    }
    
    if ("UnMHTInfoPanel" in this._window) {
      delete this._window.UnMHTInfoPanel;
    }
  },
  
  /* ==== private ==== */
  
  _icon : null,           /* nsIDOMXULElement  情報アイコン */
  _panel : null,          /* nsIDOMXULElement  情報パネル */
  _currentLocation : "",  /* String  情報パネルに表示されている本来のアドレス */
  _currentSavedFrom : "", /* String  情報パネルに表示されている保存元 */
  _linkable : false,      /* Boolean  保存元がリンク可能か */
  
  /**
   * 情報アイコンの状態を状況に合わせて切り替える
   */
  _setInfoIconState : function () {
    let targetDocument = UnMHTWindowUtil.getCurrentDocument (this._window);
    
    let location = targetDocument.location.href;
    if (UnMHTScheme.isUnMHTURI (location)) {
      let icons = this.el ("urlbar-icons");
      
      if (!this._icon) {
        this._icon = this._window.document.createElement ("image");
        this._icon.id = "unmht-urlbar-icon";
        this._icon.setAttribute ("class", "urlbar-icon");
        this._icon.addEventListener
          ("click", this.onInfoIconClick, false);
      }
      
      if (this._icon.parentNode != icons) {
        icons.insertBefore (this._icon, icons.firstChild);
      }
      
      let [eFileInfo, part] = UnMHTExtractor.getFileInfoAndPart (location);
      if (eFileInfo && part) {
        this._icon.hidden = false;
        return;
      }
    }
    
    if (this._icon) {
      this._icon.hidden = true;
    }
  }
};
UnMHTPrototype.createInheritance (UnMHTInfoPanelClass, UnMHTWindowHook);
