/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTArrayUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMap"];

/**
 * 汎用マップ
 */
function UnMHTMap () {
  this._v = [];
}
UnMHTMap.prototype = {
  /**
   * 破棄
   */
  destruct : function () {
    this._v = null;
  },
  
  /**
   * 要素を持つかどうか返す
   *
   * @param  Any key
   *         キー
   * @return Boolean
   *         要素を持つかどうか
   */
  has : function (key) {
    return this._v.some (function ([k, v]) k == key);
  },
  
  /**
   * 要素を取得する
   *
   * @param  Any key
   *         キー
   * @return Any
   *         要素
   *         無い場合は null
   */
  get : function (key) {
    let [, value]
    = UnMHTArrayUtil.find
    (this._v,
     function ([k, v]) k == key,
     [null, null]);
    
    return value;
  },
  
  /**
   * 要素を追加する
   *
   * @param  Any key
   *         キー
   * @param  Any value
   *         要素
   */
  add : function (key, value) {
    this.remove (key);
    this._v.unshift ([key, value]);
  },
  
  /**
   * 要素を削除する
   *
   * @param  Any key
   *         キー
   */
  remove : function (key) {
    this._v = this._v.filter (function ([k, v]) k != key);
  },
  
  /**
   * 要素を全て削除する
   */
  clear : function () {
    this._v = [];
  },
  
  /* ==== private ==== */
  
  _v : null /* Array  [[Any, Any], ...] */
};
