/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTPref.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMaxRunTime"];

/**
 * スクリプト実行時間管理
 */
let UnMHTMaxRunTime = {
  /**
   * 展開時のスクリプトの実行時間を設定する
   */
  setExtractTime : function  () {
    this._setTime ("open.");
  },
  
  /**
   * 保存時のスクリプトの実行時間を設定する
   */
  setSaveTime : function () {
    this._setTime ("save.");
  },

  /**
   * スクリプトの実行時間を元に戻す
   */
  restoreTime : function () {
    if (this._defaultTime == -2) {
      return;
    }
    
    let name = "dom.max_chrome_script_run_time";
    if (this._defaultTime == -1) {
      UnMHTPref.prefBranch.clearUserPref (name);
    }
    else {
      UnMHTPref.prefBranch.setIntPref (name,this._defaultTime);
    }
  },
  
  /* ==== private ==== */
  
  _defaultTime : -2, /* Number  設定前の値
                     *   -2: 未取得
                     *   -1: 設定値は無い 
                     *   他: 設定前の値 */

  /**
   * スクリプトの実行時間を設定する
   *
   * @param  String type
   *         対象の設定
   */
  _setTime : function (type) {
    if (!UnMHTPref.getPref (type + "max_run_time")) {
      return;
    }
    
    let name = "dom.max_chrome_script_run_time";
    
    if (this._defaultTime == -2) {
      if (UnMHTPref.prefBranch.prefHasUserValue (name)) {
        this._defaultTime = UnMHTPref.prefBranch.getIntPref (name);
      }
      else {
        this._defaultTime = -1;
      }
    }
    
    UnMHTPref.prefBranch.setIntPref
    (name, UnMHTPref.getPref (type + "max_run_time.time"));
  }
};
