/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;
const Cu = Components.utils;
Cu.import ("resource://gre/modules/Services.jsm");
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTAppInfo.jsm");

let EXPORTED_SYMBOLS = ["UnMHTMhtmlProtocolHandler"];

/**
 * mhtml プロトコルを通常の URI に置き換える
 *
 * @implements nsIProtocolHandler
 */
function UnMHTMhtmlProtocolHandler () {
}
UnMHTMhtmlProtocolHandler.prototype = {
  classID:          Components.ID ("{6a0b6c61-7f15-4b42-af62-da7cfcdc04a8}"),
  contractID:       "@mozilla.org/network/protocol;1?name=mhtml",
  classDescription: "UnMHT Mhtml Protocol Handler JS Component",
  QueryInterface:   XPCOMUtils.generateQI ([Ci.nsIProtocolHandler]),
  
  /* ==== nsIProtocolHandler インターフェース ==== */
  
  scheme : "mhtml", /* String  プロトコルスキーム */
  defaultPort : -1, /* Number  ポート番号
                     *         ネットワークベースではないので -1 */
    
  /* Number プロトコルの詳細
   *        file と同様のプロトコル */
  protocolFlags: (Ci.nsIProtocolHandler.URI_NOAUTH
                  | Ci.nsIProtocolHandler.URI_LOADABLE_BY_ANYONE),
  
  /**
   * ブラックリストのポートを上書きするか
   *   nsIProtocolHandler.allowPort
   *
   * @param  Number port
   *         ポート番号
   * @param  String scheme
   *         プロトコルスキーム
   * @return Boolean
   *         上書きするか
   */
  allowPort: function (port, scheme) {
    return false;
  },
    
  /**
   * URI を作成する
   *   nsIProtocolHandler.newURI
   *
   * @param  String spec
   *         対象の URI
   * @param  String charset
   *         対象の文字コード
   * @param  nsIURI baseURI
   *         読み込み元の URI
   * @return nsIURI
   *         作成した URI
   */
  newURI : function (spec, charset, baseURI) {
    /* mhtml:http://hoge
     * mhtml://http://hoge
     *   to
     * http://hoge
     */
    if (spec.match (/^mhtml:(\/\/)?([^\/]+):\/\//)) {
      return Services.io.newURI
      (spec.replace (/^mhtml:(\/\/)?([^\/]+):\/\//, "$2://"), null, null);
    }
    else {
      return Services.io.newURI ("about:blank", null, null);
    }
  },
    
  /**
   * チャネルを作成する
   *   nsIProtocolHandler.newChannel
   *
   * @param  nsIURI uri
   *         対象の URI
   * @return nsIChannel
   *         作成したチャネル
   */
  newChannel : function (uri) {
    return Services.io.newChannel ("about:blank", null, null);
  }
};
