/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTDOMUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTExtractor.jsm");
XPCOMUtils.importRelative (this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative (this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative (this, "UnMHTStringBundle.jsm");
XPCOMUtils.importRelative (this, "UnMHTTimer.jsm");

let EXPORTED_SYMBOLS = ["UnMHTPageInfo"];

/**
 * ページ情報ダイアログ
 * MHT に関する情報を追加する
 *
 * @param  nsIDOMWindow window
 *         対象のウィンドウ
 * @implements IUnMHTWindowHook
 */
function UnMHTPageInfo (window) {
  this._window = window;
  this._rows = [];
}
UnMHTPageInfo.prototype = {
  /* ==== IUnMHTWindowHook インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct : function () {
    this._window = null;
    this._rows = null;
  },
  
  /**
   * ウィンドウにロードする
   *
   * @param  Boolean isStartup
   *         スタートアップイベントか
   * @param  Number reason
   *         スタートアップの理由
   */
  load : function (isStartup, reason) {
    if (!("gDocument" in this._window
          && this._window.gDocument)) {
      UnMHTTimer.setTimeout (function () {
          this.load (isStartup, reason);
        }.bind (this), 100);
      return;
    }
    
    let urltext = this._window.document.getElementById ("urltext");
    if (!urltext) {
      return;
    }
    
    let urlRow = UnMHTDOMUtil.findParentNode (urltext, "row");
    if (!urlRow) {
      return;
    }
    
    let url = this._window.gDocument.location.toString ();
    let [eFileInfo, part] = UnMHTExtractor.getFileInfoAndPart (url);
    if (!(eFileInfo && part)) {
      return;
    }
    
    let nextSibling = urlRow.nextSibling;
    
    let addRow = function (labelText, valueText) {
      let row = this._window.document.createElement ("row");
    
      let label = this._window.document.createElement ("label");
      label.setAttribute ("control", "unmht-original-url");
      label.setAttribute ("value", labelText);
      row.appendChild (label);
    
      let sep = this._window.document.createElement ("separator");
      row.appendChild (sep);
    
      let textbox = this._window.document.createElement ("textbox");
      textbox.setAttribute ("readonly", "unmht-original-url");
      textbox.setAttribute ("id", "unmht-original-url");
      textbox.setAttribute ("value", valueText);
      row.appendChild (textbox);
      
      urlRow.parentNode.insertBefore (row, nextSibling);
      
      this._rows.push (row);
    }.bind (this);
    
    addRow (UnMHTStringBundle.get ("original_address"),
            eFileInfo.original);
    if (eFileInfo.startPart
        && eFileInfo.startPart.contentLocation) {
      addRow (UnMHTStringBundle.get ("saved_from"),
              part.contentLocation
              || eFileInfo.startPart.contentLocation);
    }
    if (eFileInfo.date) {
      let savedTime = eFileInfo.date;
      try {
        savedTime = UnMHTFormatter.getReadableTime
          ((new Date (savedTime)).getTime ());
      }
      catch (e) {
      }
          
      addRow (UnMHTStringBundle.get ("saved_date"),
              savedTime);
    }
  },
  
  /**
   * ウィンドウからアンロードする
   *
   * @param  Boolean isShutdown
   *         シャットダウンイベントか
   * @param  Number reason
   *         シャットダウンの理由
   */
  unload : function (isShutdown, reason) {
    if (this._rows) {
      this._rows.forEach (function (row) {
          if (row.parentNode) {
            row.parentNode.removeChild (row);
          }
        }.bind (this));
      this._rows = null;
    }
  },
  
  /* ==== private ==== */
  
  _window : null, /* nsIDOMWindow  対象のウィンドウ */
  _rows : null    /* Array 追加した行
                   *   [nsIDOMXULElement, ...] */
};
