/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;

let EXPORTED_SYMBOLS = ["UnMHTPrototype"];

/**
 * プロトタイプの継承を簡略化する
 */
let UnMHTPrototype = {
  ERROR_NO_SUPER_FOUND : 0x0001,
  
  /**
   * 継承したプロトタイプを作成する
   *
   * @param  Function child
   *         子
   * @param  Function parent
   *         親
   */
  createInheritance : function (child, parent) {
    let prop = {};
    for (let [name, value] in Iterator (child.prototype)) {
      prop [name] = {
        value        : value,
        enumerable   : true,
        configurable : true,
        writable     : true
      };
    }
    child.prototype = Object.create (new parent, prop);
  },
  
  /**
   * 親のプロパティを取得する
   *
   * @param  Object obj
   *         対象のオブジェクト
   * @param  String name
   *         プロパティ名
   * @return Any
   *         親のプロパティ
   * @throws UnMHTPrototype.ERROR_NO_SUPER_FOUND
   */
  super : function (obj, name) {
    let o = Object.getPrototypeOf (obj);
    
    while (o) {
      o = Object.getPrototypeOf (o);
      if (name in o) {
        return o [name];
      }
    }
    
    throw this.ERROR_NO_SUPER_FOUND;
  }
};
