/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/Services.jsm");
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTAppInfo.jsm");
XPCOMUtils.importRelative (this, "UnMHTArrayUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTCSSUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTCustomFilename.jsm");
XPCOMUtils.importRelative (this, "UnMHTDOMUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTExtractor.jsm");
XPCOMUtils.importRelative (this, "UnMHTFileUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTFormatter.jsm");
XPCOMUtils.importRelative (this, "UnMHTHTMLUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTMIMEUtil.jsm");
XPCOMUtils.importRelative (this, "UnMHTMailPrivacy.jsm");
XPCOMUtils.importRelative (this, "UnMHTMaxRunTime.jsm");
XPCOMUtils.importRelative (this, "UnMHTNetFileLoader.jsm");
XPCOMUtils.importRelative (this, "UnMHTPref.jsm");
XPCOMUtils.importRelative (this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative (this, "UnMHTShutdown.jsm");
XPCOMUtils.importRelative (this, "UnMHTStringBundle.jsm");
XPCOMUtils.importRelative (this, "UnMHTTimer.jsm");
XPCOMUtils.importRelative (this, "UnMHTTreeScanner.jsm");
XPCOMUtils.importRelative (this, "UnMHTUnicodeConverter.jsm");
XPCOMUtils.importRelative (this, "UnMHTVersion.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTSave"];

/**
 * 保存するファイルの情報
 *
 * @param  String loc
 *         保存するファイルの URI
 * @param  UnMHTSaveContext context
 *         保存コンテキスト
 * @implements IUnMHTNetFileLoaderListener
 */
function UnMHTSaveParam (loc, context) {
  this.location = loc;
  this.savedFrom = loc;
  this.currentLocation = loc;
  this.saveContext = context;
  
  this.status = UnMHTSave.STATUS_FILE_NA;
  
  let loc = this.currentLocation;
  if (UnMHTScheme.isUnMHTURI (loc)) {
    loc = UnMHTScheme.deUnMHTURI (loc);
  }
  if (loc.match (/^file:\/\//)) {
    this.isLocal = true;
  }
  
  if (UnMHTScheme.isUnMHTURI (this.currentLocation)) {
    let [eFileInfo, part]
      = UnMHTExtractor.getFileInfoAndPart (this.currentLocation);
    if (eFileInfo && part) {
      this.eFileInfo = eFileInfo;
      if (part.contentLocation) {
        this.location = part.contentLocation;
      }
      else {
        this.location = "";
        this.cid = part.eParam.cid;
      }
    }
  }
}
UnMHTSaveParam.prototype = {
  eFileInfo : null,    /* UnMHTExtractFileInfo  保存元のファイルの展開情報 */
  saveContext : null,  /* UnMHTSaveContext  保存コンテキスト */
  
  /* ファイル取得 */
  startTime : 0,        /* Number  開始時間 */
  status : 0,           /* Number  取得状況
                         *   UnMHTSave.STATUS_* */
  depth : 0,            /* Number 参照の深さ */
  
  host : "",            /* String  保存するファイルのホスト
                         *         ホスト当たりの接続数チェック用 */
  
  fileSize : 0,         /* Number  取得前に分かっているファイルサイズ
                         *         不明の場合 0 */
  recvSize : 0,         /* Number  取得したファイルサイズ */
  
  loadContext : null,   /* UnMHTNetFileLoadContext  ファイルロード
                         *                          コンテキスト */
  loaderStatus : 0,     /* Number  ファイルローダの取得状況
                         *   UnMHTNetFileLoader.STATUS_* */
  
  /* ドキュメント取得 */
  isDocument : false,      /* Boolean  ドキュメントか */
  targetDocument : null,   /* nsIDOMDocument  現在のドキュメント */
  targetSelection : null,  /* nsISelection  対象の選択範囲 */
  saveState : 0,           /* Number  保存状態 */
  hasBaseElement : false,  /* Boolean  BASE 要素を 持っているか */
  treeScanner : null,      /* UnMHTTreeScanner  ツリースキャナ */
  
  /* ファイル情報 */
  isLocal : false,      /* Boolean  ローカルファイルか */
  
  baseDir : "",         /* String  ベースディレクトリ */
  isCSS : false,        /* Boolean  CSS かどうか
                         *          CSS の import をチェックするか */
  isHTML : false,       /* Boolean  HTML かどうか
                         *          コメント, リンクを入れるか
                         *          CSS の import をチェックするか */
  isIFrame : false,     /* Boolean  インラインフレームか
                         *          コメント, リンクを入れるか */
  isText : false,       /* Boolean  テキストかどうか
                         *          quoted-printable で保存するか */
  isMail : false,       /* Boolean  メールかどうか */
  
  attachments : null,   /* Array  添付ファイル
                         *   [[String ファイル名, String URL], ...] */
  
  location : "",        /* String  Content-Location に格納する値 */
  savedFrom : "",       /* String  保存元のアドレス */
  currentLocation : "", /* String  保存対象の URI */
  cid : "",             /* String  Content-ID に格納する値 */
  referrer : "",        /* String  リファラ */
  
  content : "",         /* String  ファイルの中身 */
  
  mimetype : "",        /* String  MIME-Type
                         *         データ取得途中に変化する可能性がある */
  charset : "",         /* String  キャラクタセット
                         *         データ取得途中に変化する可能性がある */
  
  importFiles : null,   /* Array  @import しているファイル
                         *   [[String  URL,
                         *     Number nsIContentPolicy の対象の種類], ...] */
  
  /**
   * 破棄
   */
  destruct : function () {
    this.saveContext = null;
    
    this.eFileInfo = null;
    this.importFiles = null;
    
    this.targetDocument = null;
    this.targetSelection = null;
    if (this.treeScanner) {
      try {
        this.treeScanner.destruct ();
      }
      catch (e) {
      }
      this.treeScanner = null;
    }
    
    this.content = "";
  },
  
  /* ==== IUnMHTNetFileLoaderListener インターフェース ==== */
  
  /**
   * 状態変化のイベント
   *
   * @param  UnMHTNetFileLoadContext context
   *         ファイルロードコンテキスト
   * @param  Number status
   *         状態
   *           UnMHTNetFileLoader.STATUS_*
   */
  onStatusChange : function (context, status) {
    this.loaderStatus = status;
  },
    
  /**
   * 取得開始のイベント
   *
   * @param  UnMHTNetFileLoadContext context
   *         ファイルロードコンテキスト
   * @param  Number status
   *         状態
   *           UnMHTNetFileLoader.STATUS_*
   * @param  nsIURI location
   *         取得中の URI
   * @param  Number size
   *         ファイルサイズ
   *         不明ならば 0
   */
  onStart : function (context, status, location, size) {
    this.loaderStatus = status;
    this.currentLocation = location.spec;
    this.fileSize = size;    
    
    this.startTime = (new Date ()).getTime ();
    this.content = "";
  },
  
  /**
   * 取得終了のイベント
   *
   * @param  UnMHTNetFileLoadContext context
   *         ファイルロードコンテキスト
   * @param  Number status
   *         状態
   *           UnMHTNetFileLoader.STATUS_*
   * @param  nsIChannel channel
   *         ネットワークから取得した場合のチャネル
   */
  onStop : function (context, status, channel) {
    if (this.status == UnMHTSave.STATUS_NG) {
      return;
    }
    
    if (status == UnMHTNetFileLoader.STATUS_NG) {
      this.status = UnMHTSave.STATUS_NG;
      return;
    }
    
    if (this.mimetype == "" && channel) {
      let contentType = "";
      try {
        contentType
          = channel.QueryInterface (Ci.nsIHttpChannel)
          .getResponseHeader ("Content-Type");
      }
      catch (e) {
      }
      if (contentType) {
        let param
        = UnMHTMIMEUtil.decodeParam (contentType)
        if (param.type && param.subtype) {
          this.mimetype = param.type + "/" + param.subtype;
        }
        if ("charset" in param.params) {
          this.charset  = param.params ["charset"];
        }
      }
    }
    
    if (this.mimetype == "") {
      let re = this.currentLocation.match (/\.([^\.]+)(?:\?.+)?(?:#.+)?$/i);
      if (re) {
        this.mimetype = UnMHTMIMEUtil.getTypeFromExtension (re [1]);
      }
    }
    
    if (this.isHTML || this.isCSS) {
      this.importFiles
      = UnMHTCSSUtil.gatherImport
      (this.content,
       this.isHTML
       ? (this.baseDir || this.currentLocation)
       : this.currentLocation)
      .map (function (url) [url, Ci.nsIContentPolicy.TYPE_STYLESHEET]);
    }
    
    this.status = UnMHTSave.STATUS_OK;
    
    UnMHTSave.modifyContent (this, this.currentLocation,
                             this.saveContext._saveDate);
  },
  
  /**
   * データ到着のイベント
   *
   * @param  UnMHTNetFileLoadContext context
   *         ファイルロードコンテキスト
   * @param  Number status
   *         状態
   *           UnMHTNetFileLoader.STATUS_*
   * @param  String data
   *         データ
   * @param  Number count 
   *         データの長さ
   */
  onData : function (context, status, data, count) {
    this.content += data;
    this.recvSize += count;
  }
};

/**
 * 保存コンテキスト
 *
 * @param  nsIDOMWindow window
 *         実行中のウィンドウ
 * @param  nsIFile targetFile
 *         対象のファイル,
 * @param  String location
 *         メールを保存する際のルートドキュメントの URI
 * @param  String subject
 *         ファイルの subject
 * @param  IUnMHTSaveListener listener
 *         イベントリスナ
 * @return UnMHTSaveContext
 *         保存コンテキスト
 */
function UnMHTSaveContext (window, targetFile, location, subject, listener) {
  this._window = window;
  this._targetFile = targetFile;
  this._location = location;
  this._listener = listener;
  
  this._saveDate = new Date ();
  
  this._urls = {};
  this._saveParams = [];
  
  this._deepImportAction = UnMHTPref.getPref ("save.deep_import.action");
  
  let name = "network.http.max-connections";
  if (UnMHTPref.prefBranch.getPrefType (name) != 0) {
    this._maxConns = Math.ceil (UnMHTPref.prefBranch.getIntPref (name) / 2);
  }
  name = "network.http.max-connections-per-server";
  if (UnMHTPref.prefBranch.getPrefType (name) != 0) {
    this._maxConnsPerServer
      = Math.ceil (UnMHTPref.prefBranch.getIntPref (name) / 2);
  }
  
  let saveContentsAction = {};
  ["video", "embed"].forEach (function (type) {
      saveContentsAction [type] = UnMHTPref.getPref ("save.contents." + type);
    });
  this._saveContentsAction = saveContentsAction;
  
  this._prepareTopPart (subject);
}
UnMHTSaveContext.prototype = {
  progress : 0,           /* Number  進捗状況 (0-1) */
  
  /**
   * 破棄処理
   * start 後は自分で呼ぶ
   */
  destruct : function () {
    if (this._saveParams) {
      this._saveParams
      .forEach (function (sParam) sParam.destruct ());
      this._saveParams = null;
    }
    
    if (this._topPart) {
      UnMHTMIMEUtil.destructPart (this._topPart);
      this._topPart = null;
    }
    
    if (this._mailPart) {
      UnMHTMIMEUtil.destructPart (this._mailPart);
      this._mailPart = null;
    }
    
    this._targetFile = null;
    this._urls = null;
    this._startPart = null;
    this._listener = null;
    
    this._window = null;
  },
  
  /**
   * フィールドを追加する
   *
   * @param  String name
   *         フィールド名
   * @param  String vlaue
   *         フィールド値
   */
  addField : function (name, value) {
    if (!this._isNormalField (name)) {
      return;
    }
    if (name.toLowerCase () == "content-type") {
      return;
    }
    UnMHTMIMEUtil.addField (this._topPart, name, value);
  },
  
  /**
   * 複数のタブを含むかどうかを指定する
   *
   * @param  Boolean isTabs
   *         複数のタブを含むかどうか
   */
  setTabs : function (isTabs) {
    this._isTabs = isTabs;
  },
  
  /**
   * ドキュメントを保存対象に追加する
   *
   * @param  Object arg
   *         {
   *           targetDocument  : nsIDOMDocument  対象のドキュメント,
   *           targetSelection : nsISelection  対象の選択範囲もしくは null,
   *           scrapBookData   : Object  ScrapBook のデータ,
   *           saveState       : Number  保存状態,
   *           location        : String  ドキュメントの URL,
   *           isHTML          : Boolean HTML  かどうか,
   *           isStartPart     : Boolean  開始パートか,
   *           savedFrom       : String  保存元 (オプショナル),
   *           window          : nsIDOMWindow  ドキュメントを持つブラウザを
   *                                           実行中のウィンドウ
   *                                           メールの添付ファイル取得に使う
   *                                           (オプショナル),
   *         }
   * @return UnMHTMIMEPart
   *         対応するパート
   *         失敗したら null
   */
  addDocument : function (arg) {
    if (this._isStopped || this._isCanceled) {
      return null;
    }
    
    let getArg = function (name, defValue) {
      return (name in arg) ? arg [name] : defValue;
    };
    
    let targetDocument  = getArg ("targetDocument", null);
    let targetSelection = getArg ("targetSelection", null);
    let scrapBookData   = getArg ("scrapBookData", null);
    let saveState       = getArg ("saveState", UnMHTSave.SAVESTATE_CURRENT);
    let location        = getArg ("location", "");
    let isHTML          = getArg ("isHTML", false);
    let isStartPart     = getArg ("isStartPart", false);
    let window          = getArg ("window", null);
    
    let savedFrom       = getArg ("savedFrom", location);
    
    UnMHTMaxRunTime.setSaveTime ();
    
    let part = null;
    if (scrapBookData) {
      part = this._addScrapBook (scrapBookData, isStartPart);
    }
    else if ((saveState & UnMHTSave.SAVESTATE_MASK_ORIGINAL)
             && UnMHTScheme.isUnMHTURI (location)) {
      part = this._addUnMHTDocument (location, isStartPart, savedFrom);
    }
    else if (isHTML) {
      part
      = this._addNormalDocument
      (targetDocument, targetSelection,
       saveState,
       location, isStartPart,
       0, false, window);
    }
    else {
      part = this.addFile (targetDocument, location, isStartPart);
    }
    
    UnMHTMaxRunTime.restoreTime ();
    
    if (!part) {
      this._isStopped = true;
      this._listener.onError (this, UnMHTSave.ERROR_ADD_DOCUMENT, null);
      return null;
    }
    
    if (part.sParam) {
      part.sParam.savedFrom = savedFrom;
    }
    
    return part;
  },
  
  /**
   * データドキュメントを保存対象に追加する
   *
   * @param  String content
   *         内容
   * @param  String location
   *         Content-Location の値
   * @param  String charset
   *         Content-Type のうち、文字コード
   * @param  String mimetype
   *         Content-Type のうち、MIME タイプ
   * @param  Boolean isStartPart
   *         開始パートか
   * @return UnMHTMIMEPart
   *         対応するパート
   *         失敗したら null
   */
  addDataDocument : function (content, location, charset, mimetype,
                              isStartPart) {
    let part = new UnMHTMIMEPart ();
    let sParam = new UnMHTSaveParam (location, this);
    part.sParam = sParam;
    sParam.depth = 0;
    
    sParam.charset = charset;
    sParam.mimetype = mimetype;
    switch (sParam.mimetype) {
      case "text/html": {
        sParam.isHTML = true;
        sParam.isText = true;
        break;
      }
      case "text/css": {
        sParam.isCSS = true;
        sParam.isText = true;
        break;
      }
      case "text/javascript": {
        sParam.isText = true;
        break;
      }
    }
    
    sParam.content = content;
    
    sParam.status = UnMHTSave.STATUS_OK;
    
    this._saveParams.push (sParam);
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift (part);
    }
    else {
      this._topPart.parts.push (part);
    }
    
    return part;
  },
  
  /**
   * ファイルを保存対象に追加する
   *
   * @param  nsIDOMDocument currentDocument
   *         対象のファイルを開いたドキュメント
   *         なければ null
   * @param  String location
   *         対象のファイルの URL
   * @param  Boolean isStartPart
   *         開始パートか
   * @return UnMHTMIMEPart
   *         対応するパート
   *         失敗したら null
   */
  addFile : function (currentDocument, location, isStartPart) {
    if (!UnMHTSave.checkPolicy (Ci.nsIContentPolicy.TYPE_OBJECT,
                                location, currentDocument)) {
      return;
    }
    
    let urlForCheck = UnMHTMailPrivacy.hideMailURL (location, this._location);
    this._urls [urlForCheck] = true;
    
    let part = new UnMHTMIMEPart ();
    let sParam = new UnMHTSaveParam (location, this);
    part.sParam = sParam;
    if (currentDocument) {
      try {
        if ("referrer" in currentDocument
            && currentDocument.referrer) {
          sParam.referrer = currentDocument.referrer;
        }
        
        sParam.mimetype = currentDocument.contentType;
        sParam.charset = currentDocument.characterSet || "utf-8";
      }
      catch (e) {
      }
    }
    
    this._saveParams.push (sParam);
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift (part);
    }
    else {
      this._topPart.parts.push (part);
    }
    
    return part;
  },
  
  /**
   * 保存開始
   */
  start : function () {
    UnMHTMaxRunTime.setSaveTime ();
    
    if (this._saveParams.length > 0) {
      UnMHTTimer.setTimeout
        (function () {
          this._checkProgress ();
        }.bind (this), 100);
    }
    else {
      this.progress = 1;
      this._listener.onProgress (this, "", "0 / 0", 0, 0, 0);
      
      UnMHTTimer.setTimeout
      (function () {
        this._storeToFile ();
      }.bind (this), 100);
    }
  },
  
  /**
   * 保存をキャンセル
   */
  cancel : function () {
    if (this._isStopped) {
      this._listener.onCancel (this);
      return;
    }
    
    this._isCanceled = true;
    
    if (this._saveParams) {
      this._saveParams.forEach (function (sParam) {
          try {
            sParam.loadContext.cancel ();
          }
          catch (e) {
          }
        });
    }
  },
  
  /* ==== private ==== */
  
  _window : null,          /* nsIDOMWindow 実行中のウィンドウ */
  _targetFile : null,      /* nsIFile  対象のファイル */
  _location : "",          /* String  ルートドキュメントの URI */ 
  _listener : null,        /* IUnMHTSaveListener  イベントリスナ */
  
  _maxConns : 30,          /* Number  最大リクエスト数 */
  _maxConnsPerServer : 8,  /* Number  サーバあたりの最大リクエスト数 */
  
  _saveDate : null,        /* Date 保存開始時刻 */
  
  _topPart : null,         /* UnMHTMIMEPart  トップレベルのパート */
  _startPart : null,       /* UnMHTMIMEPart  開始パート */
  _isMixed : false,        /* Boolean  トップレベルが multipart/mixed か */
  _isAlternative : false,  /* Boolean  トップレベルが
                            *          multipart/alternative か */
  _isTabs : false,         /* Boolean 複数のタブを含むかどうか */
  
  _urls : null,            /* Object  収集した URL */
  _saveParams : null,      /* Array 作業中の保存情報
                            *   [UnMHTSaveParam, ...] */
  
  _isCanceled : false,     /* Boolean  キャンセルされたか */ 
  _isStopped : false,      /* Boolean  保存が停止したか */
  
  _deepImportAction : "",  /* String  現在の値
                            *   "save": 保存する
                            *   "ignore": 無視する
                            *   "ask": ダイアログを表示する */
  
  _saveContentsAction : "", /* Object  コンテンツの保存の選択
                             *   <String 種類, String 保存するか>
                             *     "save": 保存する
                             *     "ignore": 無視する
                             *     "ask": ダイアログを表示する */
  
  _isMailOriginal : false,  /* Boolean  メールを元のファイルで保存するか */
  
  /**
   * メールのデータ取得完了
   *
   * @param  Number status
   *         取得状況
   *           UnMHTNetFileLoader.STATUS_*
   * @param  String data
   *         データ
   * @param  UnMHTSaveParam sParam
   *         ドキュメントのファイルの情報
   */
  _onMailData : function (status, data, sParam) {
    let mailPart = null;
    if (status != UnMHTNetFileLoader.STATUS_NG) {
      mailPart
        = UnMHTMIMEUtil.decodeMessage
        (data.replace (/\r\n|\r|\n/g, "\r\n"), true);
    }
    
    if (mailPart
        && UnMHTPref.getPref ("save.header.mail.original_field")) {
      this._addFieldList (mailPart.fieldList);
    }
    
    if (sParam.status == UnMHTSave.STATUS_MAIL_CUR_MAIL) {
      sParam.status = UnMHTSave.STATUS_DOC_CUR_NA;
      this._mailPart = mailPart;
    }
    else if (sParam.status == UnMHTSave.STATUS_MAIL_ORI_MAIL) {
      if (mailPart) {
        mailPart.fieldList
        = mailPart.fieldList
        .filter (function ([name, value]) {
            let n = name.toLowerCase ();
            return (n == "content-type"
                    || n == "content-transfer-encoding");
          });
        UnMHTMIMEUtil.createFieldsFromList (mailPart);
        
        this._topPart.parts.push (mailPart);
        
        sParam.status = UnMHTSave.STATUS_MAIL_ORI_NA;
      }
      else {
        sParam.status = UnMHTSave.STATUS_NG;
      }
    }
  },
  
  /**
   * 通常のパートのフィールドかどうか調べる
   *
   * @param  String name
   *         フィールド名
   * @return Boolean
   *         通常のパートのフィールドかどうか
   */
  _isNormalField : function (name) {
    name = name.toLowerCase ();
    
    return !(name == "from"
             || name == "subject"
             || name == "date"
             || name == "mime-version"
             || name == "x-unmht-tabs");
  },
  
  /**
   * ファイルを追加可能かどうかチェックして追加する
   *
   * @param  String url
   *         対象の URL
   * @param  Number type
   *         nsIContentPolicy の対象の種類
   * @param  nsIDOMDocument currentDocument
   *         対象のドキュメント
   * @param  Number depth
   *         現在の深さ
   * @param  String referrer
   *         リファラ
   * @param  String mimetype
   *         ファイルの MIME-Type
   *         オプショナル
   */
  _checkAndPush : function (url, type,
                            currentDocument,
                            depth, referrer,
                            mimetype) {
    if (!url) {
      return;
    }
    
    if (url.match (/^data:/)) {
      /* data スキームは別途保存する必要が無い */
      return;
    }
    
    if (this._isMailOriginal) {
      if (UnMHTMailPrivacy.isMailURL (url)) {
        return;
      }
      if (url.match (/^chrome:/)) {
        return;
      }
    }
    
    if (!mimetype
        && UnMHTMailPrivacy.isMailURL (url)) {
      let re = url.match (/(^.+\?.*)&type=([^&]+)(.*)/);
      if (re) {
        mimetype = re [2];
      }
    }
    
    let urlForCheck = UnMHTMailPrivacy.hideMailURL (url, this._location);
    if (!(urlForCheck in this._urls)
        && UnMHTSave.checkPolicy (type, url, currentDocument)) {
      this._urls [urlForCheck] = true;
      
      let part = new UnMHTMIMEPart ();
      let sParam = new UnMHTSaveParam (url, this);
      part.sParam = sParam;
      
      sParam.depth = depth + 1;
      sParam.referrer = referrer;
      
      switch (type) {
        case Ci.nsIContentPolicy.TYPE_STYLESHEET: {
          sParam.isCSS = true;
          sParam.isText = true;
          sParam.mimetype = "text/css";
          break;
        }
        case Ci.nsIContentPolicy.TYPE_SCRIPT: {
          sParam.isText = true;
          sParam.mimetype = "text/javascript";
          break;
        }
      }
      
      if (mimetype) {
        sParam.mimetype = mimetype;
      }
      
      this._saveParams.push (sParam);
      this._topPart.parts.push (part);
    }
  },
  
  /**
   * 対象の要素から保存対象のファイルの URL を収集する
   *
   * @param  nsIDOMDocument currentDocument
   *         対象のドキュメント
   * @param  String location
   *         ドキュメントの URL
   * @param  Number saveState
   *         保存状態
   * @param  nsIDOMElement node
   *         対象の要素
   * @param  String baseDir
   *         ベースディレクトリ
   * @param  Number depth
   *         現在の深さ
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @param  String referrer
   *         リファラ
   */
  _gatherFilesFromNode : function (currentDocument, location,
                                   saveState,
                                   node, baseDir, depth,
                                   targetSelection,
                                   referrer, isMail) {
    if (targetSelection) {
      if (!UnMHTDOMUtil.isNodeInSelection (node, targetSelection)) {
        return;
      }
    }
    
    let style = null;
    try {
      style = currentDocument.defaultView.getComputedStyle (node, "");
    }
    catch (e) {
    }
    if (style) {
      let urls = style.getPropertyCSSValue ("background-image");
      if (urls instanceof Ci.nsIDOMCSSValueList) {
        Array.forEach
          (urls,
           function (url) {
            if (url instanceof Ci.nsIDOMCSSValue
                && "primitiveType" in url
                && url.primitiveType == Ci.nsIDOMCSSPrimitiveValue.CSS_URI) {
              url = UnMHTFileUtil.jointPath (baseDir, url.getStringValue ());
              this._checkAndPush (url, Ci.nsIContentPolicy.TYPE_IMAGE,
                                  currentDocument, depth, referrer);
            }
          }.bind (this));
      }
    }
    
    if ("background" in node) {
      this._checkAndPush (node.background, Ci.nsIContentPolicy.TYPE_IMAGE,
                          currentDocument, depth, referrer);
    }
    
    if (node instanceof Ci.nsIDOMHTMLStyleElement) {
      UnMHTCSSUtil.gatherImport
      (UnMHTDOMUtil.convertToPlainText (location, node, null, {
          noScript : false,
          isMail : isMail,
          saveCanvas : false,
        }),
       baseDir)
      .forEach (function (url) {
          this._checkAndPush (url, Ci.nsIContentPolicy.TYPE_STYLESHEET,
                              currentDocument, depth, referrer);
        }.bind (this));
    }
    else if (node instanceof Ci.nsIDOMHTMLImageElement) {
      this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_IMAGE,
                          currentDocument, depth, referrer);
    }
    else if (node instanceof Ci.nsIDOMHTMLVideoElement) {
      if (this._checkSaveContents ("video")) {
        this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer);
        this._checkAndPush (node.poster, Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer);
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLAudioElement) {
      if (this._checkSaveContents ("video")) {
        this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer);
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLSourceElement) {
      if (this._checkSaveContents ("video")) {
        this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer);
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLLinkElement
             && node.rel) {
      if (node.rel.match (/\bicon\b/i)) {
        this._checkAndPush (node.href, Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer);
      }
      else if (node.rel.match (/\bstylesheet\b/i)) {
        this._checkAndPush (node.href, Ci.nsIContentPolicy.TYPE_STYLESHEET,
                            currentDocument, depth, referrer);
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLInputElement
             && node.type.toLowerCase () == "image") {
      this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_IMAGE,
                          currentDocument, depth, referrer);
    }
    else if (node instanceof Ci.nsIDOMHTMLScriptElement) {
      if (saveState != UnMHTSave.SAVESTATE_CURRENT_NOJS) {
        this._checkAndPush (node.src, Ci.nsIContentPolicy.TYPE_SCRIPT,
                            currentDocument, depth, referrer);
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLEmbedElement) {
      if (this._checkSaveContents ("embed")) {
        this._checkAndPush (UnMHTFileUtil.jointPath (baseDir, node.src),
                            Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer,
                            node.getAttribute ("type"));
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLObjectElement) {
      if (this._checkSaveContents ("embed")) {
        this._checkAndPush (UnMHTFileUtil.jointPath (baseDir, node.data),
                            Ci.nsIContentPolicy.TYPE_OBJECT,
                            currentDocument, depth, referrer,
                            node.getAttribute ("type"));
      }
    }
    else if (node instanceof Ci.nsIDOMHTMLFrameElement) {
      this._addNormalDocument
      (node.contentDocument, targetSelection,
       saveState,
       node.contentDocument.location.href, false,
       depth, false, null);
    }
    else if (node instanceof Ci.nsIDOMHTMLIFrameElement) {
      this._addNormalDocument
      (node.contentDocument, targetSelection,
       saveState,
       node.contentDocument.location.href, false,
       depth, true, null);
    }
  },
  
  /**
   * コンテンツを保存するかどうか調べる
   *
   * @param  String type
   *         種類
   * @return Boolean
   *         保存するか
   */
  _checkSaveContents : function (type) {
    let action = this._saveContentsAction [type];
    if (action == "ask") {
      action = UnMHTSave.showSaveContentsConfirm (this._window, type);
      this._saveContentsAction [type] = action;
    }
    
    return action == "save";
  },

  /**
   * CSS の階層が深い場合に保存するかどうか調べる
   *
   * @return Boolean
   *         保存するか
   */
  _checkDeepImportAction : function () {
    let action =this._deepImportAction;
    if (this._deepImportAction == "ask") {
      action = UnMHTSave.showDeepCSSConfirm (this._window);
      this._deepImportAction = action;
    }
    
    return action == "save";
  },
  
  /**
   * 通常の HTML ドキュメントを保存対象に追加する
   *
   * @param  nsIDOMDocument currentDocument
   *         対象のドキュメント
   * @param  nsISelection targetSelection
   *         対象の選択範囲もしくは null
   * @param  Number saveState
   *         保存状態
   * @param  String location
   *         ドキュメントの URL
   * @param  Boolean isStartPart
   *         開始パートか
   * @param  Number depth
   *         現在の深さ
   * @param  Boolean isIFrame
   *         インラインフレームか
   * @param  nsIDOMWindow window
   *         ドキュメントを持つブラウザを実行中のウィンドウ
   *         メールの添付ファイル取得に使う (オプショナル)
   * @return UnMHTMIMEPart
   *         メインのパート
   *         失敗したら null
   */
  _addNormalDocument : function (currentDocument, targetSelection,
                                 saveState,
                                 location, isStartPart,
                                 depth, isIFrame,
                                 window) {
    let urlForCheck = UnMHTMailPrivacy.hideMailURL (location, this._location);
    this._urls [urlForCheck] = true;
    
    let part = new UnMHTMIMEPart ();
    let sParam = new UnMHTSaveParam (location, this);
    part.sParam = sParam;
    
    sParam.depth = depth + 1;
    try {
      if ("referrer" in currentDocument
          && currentDocument.referrer) {
        sParam.referrer = currentDocument.referrer;
      }
    }
    catch (e) {
      UnMHTMIMEUtil.destructPart (part)
      sParam.destruct ();
      return null;
    }
    
    sParam.isIFrame = isIFrame;
    sParam.isHTML = true;
    sParam.isText = true;
    sParam.isMail = UnMHTMailPrivacy.isMailURL (location);
    sParam.mimetype = "text/html";
    try {
      sParam.charset = currentDocument.characterSet || "utf-8";
    }
    catch (e) {
      sParam.charset = "utf-8";
    }
    
    if (isStartPart && sParam.eFileInfo) {
      /* UnMHT の展開した結果であり
       * 開始パートならば展開結果のヘッダを継承する */
      this._addFieldList (sParam.eFileInfo.topPart.fieldList);
    }
    
    let baseDir = location.replace (/[^\/]+$/, "/");
    try {
      Array.forEach
      (currentDocument.getElementsByTagName ("base"),
       function (node) {
        sParam.hasBaseElement = true;
        if (node.hasAttribute ("href")
            && node.getAttribute ("href")) {
          let baseURI;
          try {
            baseURI
              = Services.io.newURI (node.getAttribute ("href"), null, null);
          }
          catch (e) {
            return;
          }
          baseDir = UnMHTFileUtil.jointPath (baseDir, baseURI.resolve ("./"));
        }
      });
    }
    catch (e) {
      UnMHTMIMEUtil.destructPart (part)
      sParam.destruct ();
      return null;
    }
    sParam.baseDir = baseDir;
    
    sParam.isDocument = true;
    sParam.targetDocument = currentDocument;
    sParam.targetSelection = targetSelection;
    sParam.saveState = saveState;
    
    if ((saveState & UnMHTSave.SAVESTATE_MASK_CURRENT)
        || targetSelection) {
      sParam.status = UnMHTSave.STATUS_DOC_CUR_NA;
    }
    else {
      sParam.status = UnMHTSave.STATUS_DOC_ORI_NA;
    }
    this._saveParams.push (sParam);
    if (isStartPart) {
      this._startPart = part;
      this._topPart.parts.unshift (part);
    }
    else {
      this._topPart.parts.push (part);
    }
    
    if (sParam.isMail && window) {
      if (sParam.status == UnMHTSave.STATUS_DOC_ORI_NA) {
        this._isMailOriginal = true;
      }
      else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_NA) {
        if ("currentAttachments" in window
            && window.currentAttachments) {
          sParam.attachments
          = window.currentAttachments.map (function (attachment) {
              if (attachment.hasFile
                  && !attachment.isDeleted
                  && !attachment.isExternalAttachment
                  && attachment.url) {
                let filePart = this.addFile (null, attachment.url, false);
                filePart.sParam.mimetype = attachment.contentType;
                return [attachment.name, attachment.url];
              }
              return null;
            }.bind (this))
          .filter (function (attachment) attachment);
        }
      }
      
      if ("gFolderDisplay" in window
          && "selectedMessageUris" in window.gFolderDisplay
          && window.gFolderDisplay.selectedMessageUris.length == 1) {
        let mailSession
        = Cc ["@mozilla.org/messenger/services/session;1"]
        .getService (Ci.nsIMsgMailSession);
          
        let messageURL
        = mailSession.ConvertMsgURIToMsgURL
        (window.gFolderDisplay.selectedMessageUris [0], window.msgWindow);
        
        let uri;
        try {
          uri = Services.io.newURI (messageURL, null, null);
        }
        catch (e) {
        }
        
        if (uri) {
          if (sParam.status == UnMHTSave.STATUS_DOC_ORI_NA) {
            sParam.status = UnMHTSave.STATUS_MAIL_ORI_MAIL;
          }
          else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_NA) {
            sParam.status = UnMHTSave.STATUS_MAIL_CUR_MAIL;
          }
          
          UnMHTNetFileLoader.loadData
          (uri, null, true, function (status, data) {
            this._onMailData (status, data, sParam);
          }.bind (this));
        }
      }
    }
    
    return part;
  },
  
  /**
   * UnMHT で展開したドキュメントを保存対象に追加する
   *
   * @param  String location
   *         対象の URI
   * @param  Boolean isStartPart
   *         開始パートか
   * @param  String savedFrom
   *         保存元のアドレス
   * @return UnMHTMIMEPart
   *         メインのパート
   *         失敗したら null
   */
  _addUnMHTDocument : function (location, isStartPart, savedFrom) {
    let [eFileInfo, currentPart]
    = UnMHTExtractor.getFileInfoAndPart (location);
    if (!(eFileInfo && currentPart)) {
      return null;
    }
    
    if (isStartPart && currentPart == eFileInfo.startPart) {
      this._addFieldList (eFileInfo.topPart.fieldList);
    }
    
    let createSaveParam = function (part) {
      if (part.isMultipart) {
        part.parts.forEach (function (p) createSaveParam (p));
        return null;
      }
      
      if (part.mimetype == "text/html") {
        let sParam = new UnMHTSaveParam (location, this);
          
        sParam.isHTML = true;
        sParam.content = part.body;
        
        UnMHTSave.modifyContent (sParam, savedFrom, this._saveDate);
        
        part.body = sParam.content;
          
        sParam.destruct ();
      }
    }.bind (this);
    
    let findPart = {
      from: currentPart,
      to: null
    };
    let part = UnMHTMIMEUtil.clonePart (eFileInfo.topPart, findPart);
    let mainPart = findPart.to;
    
    createSaveParam (part);
    
    if (isStartPart) {
      if (part.isMultipart) {
        this._startPart = UnMHTMIMEUtil.findStartPart (part);
        if (this._startPart.mimetype == "multipart/mixed") {
          this._startPart = null;
          this._isMixed = true;
        }
        else if (this._startPart.mimetype == "multipart/alternative") {
          this._startPart = null;
          this._isAlternative = true;
        }
        
        part.parts.forEach (function (p) {
            this._topPart.parts.push (p);
          }.bind (this));
      }
      else {
        this._startPart = part;
        this._topPart.parts.unshift (part);
      }
    }
    else {
      part.fieldList
      = part.fieldList.filter
      ((function ([name, value]) this._isNormalField (name)).bind (this));
      UnMHTMIMEUtil.createFieldsFromList (part);
      
      this._topPart.parts.push (part);
    }
    
    return mainPart;
  },
  
  /**
   * ScrapBook のデータを保存対象に追加する
   *
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   * @param  Boolean isStartPart
   *         開始パートか
   * @return UnMHTMIMEPart
   *         メインのパート
   *         失敗したら null
   */
  _addScrapBook : function (scrapBookData, isStartPart) {
    let mainPart = null;
    
    UnMHTFileUtil.getDirectoryEntries (scrapBookData ["dir"])
    .forEach (function (targetFile) {
        if (targetFile.leafName == "index.dat") {
          return;
        }
        
        let isIndex = false;
        if (targetFile.leafName == "index.html") {
          isIndex = true;
        }
        
        let part = new UnMHTMIMEPart ();
        let sParam = new UnMHTSaveParam (targetFile.leafName, this);
        part.sParam = sParam;
          
        sParam.depth = 0;
        sParam.referrer = "";
          
        if (isIndex) {
          sParam.location = scrapBookData ["source"];
          sParam.mimetype = "text/html";
          sParam.charset = scrapBookData ["charset"];
        }
        else {
          sParam.location
            = UnMHTFileUtil.jointPath
            (scrapBookData ["source"].replace (/\/[^\/]*$/, ""),
             sParam.location);
          let re = sParam.location.match (/\.([^\.]+)(?:\?.+)?(?:#.+)?$/i);
          if (re) {
            sParam.mimetype = UnMHTMIMEUtil.getTypeFromExtension (re [1]);
          }
        }
        sParam.savedFrom = sParam.location;
        sParam.currentLocation = sParam.location;
          
        sParam.content = UnMHTFileUtil.readData (targetFile);

        UnMHTSave.modifyContent (sParam, sParam.savedFrom, this._saveDate);
          
        sParam.status = UnMHTSave.STATUS_OK;
          
        this._saveParams.push (sParam);
        if (isIndex) {
          mainPart = part;
          this._topPart.parts.unshift (part);
          if (isStartPart) {
            this._startPart = part;
          }
        }
        else {
          this._topPart.parts.push (part);
        }
      }.bind (this));
    
    return mainPart;
  },
  
  /**
   * インポートされたファイルを追加する
   *
   * @param  UnMHTSaveParam sParam
   *         対象の保存情報
   * @param  Number na
   *         未取得のファイル数
   */
  _addImportFiles : function (sParam) {
    if ((sParam.status == UnMHTSave.STATUS_OK
         || sParam.status == UnMHTSave.STATUS_NG)
        && sParam.importFiles
        && sParam.importFiles.length > 0) {
      if (sParam.depth + 1 >= UnMHTSave.MAX_IMPORT_DEPTH) {
        if (!this._checkDeepImportAction ()) {
          return;
        }
      }
      
      sParam.importFiles.forEach (function ([url, type]) {
          let urlForCheck = UnMHTMailPrivacy.hideMailURL (url, this._location);
          if (!(urlForCheck in this._urls)
              && UnMHTSave.checkPolicy (type, url, null)) {
            this._urls [urlForCheck] = true;
              
            let part = new UnMHTMIMEPart ();
            let newSParam = new UnMHTSaveParam (url, this);
            part.sParam = newSParam;
              
            newSParam.depth = sParam.depth + 1;
            newSParam.referrer = sParam.currentLocation;
              
            if (type == Ci.nsIContentPolicy.TYPE_STYLESHEET) {
              newSParam.isText = true;
              newSParam.isCSS = true;
              newSParam.mimetype = "text/css";
            }
              
            this._saveParams.push (newSParam);
            this._topPart.parts.push (part);
          }
        }.bind (this));
      sParam.importFiles = null;
    }
  },
  
  /**
   * mht で保存 - ファイル収集のチェック
   */
  _checkProgress : function () {
    if (this._isCanceled) {
      UnMHTMaxRunTime.restoreTime ();
      this._isStopped = true;
      this._listener.onCancel (this);
      this.destruct ();
      return;
    }
    
    let ok = 0;
    let na_progress = 0;
    let ng = 0;
    
    let conns = 0;
    let connsPerServer = new Object ();
    
    let currentFile = "";
    let currentSize = "";
    
    let now = (new Date ()).getTime ();
    
    this._saveParams.forEach (function (sParam) {
        if (currentFile == ""
            && sParam.status == UnMHTSave.STATUS_FILE_NET
            && sParam.fileSize) {
          currentFile = sParam.currentLocation;
          currentSize
            = sParam.recvSize + " bytes / " + sParam.fileSize + " bytes";
        }
        
        if (currentFile == ""
            && sParam.status != UnMHTSave.STATUS_OK
            && sParam.status != UnMHTSave.STATUS_NG) {
          currentFile = sParam.currentLocation;
          currentSize
            = sParam.recvSize + " bytes / "
            + (sParam.fileSize || "unknown") + " bytes";
        }
        
        this._addImportFiles (sParam);
        
        if (sParam.status == UnMHTSave.STATUS_FILE_NET) {
          if (sParam.fileSize) {
            let progress = sParam.recvSize / sParam.fileSize;
            if (sParam.isDocument) {
              progress = 0.5 + progress * 0.5;
            }
            na_progress += progress;
          }
          
          conns ++;
          if (sParam.host in connsPerServer) {
            connsPerServer [sParam.host] ++;
          }
          else {
            connsPerServer [sParam.host] = 1;
          }
          
          if (sParam.recvSize == 0) {
            if (sParam.loaderStatus == UnMHTNetFileLoader.STATUS_NA
                || sParam.loaderStatus == UnMHTNetFileLoader.STATUS_CACHE) {
              if (now > sParam.startTime + UnMHTSave.CACHE_TIMEOUT * 1000) {
                sParam.startTime = now;
                sParam.loaderStatus = UnMHTNetFileLoader.STATUS_NET;
                
                sParam.loadContext.skipCache ();
              }
            }
            else if (sParam.loaderStatus == UnMHTNetFileLoader.STATUS_NET) {
              if (now > sParam.startTime + UnMHTSave.RECEIVE_TIMEOUT * 1000) {
                try {
                  sParam.loadContext.cancel ();
                }
                catch (e) {
                }
                
                sParam.status = UnMHTSave.STATUS_NG;
              }
            }
          }
        }
        else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_NA) {
          sParam.status = UnMHTSave.STATUS_DOC_CUR_SERIAL;
        }
        else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_SERIAL) {
          try {
            if (this._serializeDocument (sParam)) {
              sParam.treeScanner
                = new UnMHTTreeScanner (sParam.targetDocument.documentElement);
              sParam.status = UnMHTSave.STATUS_DOC_CUR_GATHER;
            }
          }
          catch (e) {
            if (UnMHTSave.isAlive (sParam.targetDocument)) {
              Cu.reportError (e + "\n" + e.stack);
            }
            sParam.status = UnMHTSave.STATUS_NG;
          }
        }
        else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_NA) {
          sParam.treeScanner
            = new UnMHTTreeScanner (sParam.targetDocument.documentElement);
          sParam.status = UnMHTSave.STATUS_DOC_ORI_GATHER;
        }
        else if (sParam.status == UnMHTSave.STATUS_MAIL_ORI_NA) {
          sParam.treeScanner
            = new UnMHTTreeScanner (sParam.targetDocument.documentElement);
          sParam.status = UnMHTSave.STATUS_MAIL_ORI_GATHER;
        }
        else if (sParam.status == UnMHTSave.STATUS_DOC_CUR_GATHER
                 || sParam.status == UnMHTSave.STATUS_DOC_ORI_GATHER
                 || sParam.status == UnMHTSave.STATUS_MAIL_ORI_GATHER) {
          try {
            if (this._gatherDocument (sParam)) {
              sParam.treeScanner.destruct ();
              sParam.treeScanner = null;
              if (sParam.status == UnMHTSave.STATUS_DOC_CUR_GATHER) {
                ok ++;
                sParam.status = UnMHTSave.STATUS_OK;
              }
              else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_GATHER) {
                na_progress += 0.5;
                sParam.status = UnMHTSave.STATUS_FILE_NA;
              }
              else if (sParam.status == UnMHTSave.STATUS_MAIL_ORI_GATHER) {
                /* 完了したらこのパートは不要 */
                sParam.status = UnMHTSave.STATUS_IGNORE;
              }
            }
            else {
              let progress = sParam.treeScanner.progress;
              if (this.progress > 1) {
                this.progress = 1;
              }
              
              if (sParam.status == UnMHTSave.STATUS_DOC_CUR_GATHER) {
                na_progress += 0.5 + progress * 0.5;
              }
              else if (sParam.status == UnMHTSave.STATUS_DOC_ORI_GATHER) {
                na_progress += progress * 0.5;
              }
              else if (sParam.status == UnMHTSave.STATUS_MAIL_ORI_GATHER) {
                na_progress += progress;
              }
            }
          }
          catch (e) {
            if (UnMHTSave.isAlive (sParam.targetDocument)) {
              Cu.reportError (e + "\n" + e.stack);
            }
            sParam.status = UnMHTSave.STATUS_NG;
          }
        }
        else if (sParam.status == UnMHTSave.STATUS_OK
                 || sParam.status == UnMHTSave.STATUS_IGNORE) {
          ok ++;
        }
        else if (sParam.status == UnMHTSave.STATUS_NG) {
          ng ++;
        }
      }.bind (this));
    
    this._saveParams.forEach (function (sParam) {
        if (sParam.status == UnMHTSave.STATUS_FILE_NA) {
          if (!sParam.host) {
            try {
              let uri = Services.io.newURI (sParam.currentLocation, null, null);
              sParam.host = uri.host;
            }
            catch (e) {
              sParam.host = "";
            }
            if (!sParam.host) {
              sParam.host = "unknown";
            }
          }
            
          if (conns >= this._maxConns) {
            return;
          }
          if (sParam.host in connsPerServer
              && connsPerServer [sParam.host] >= this._maxConnsPerServer) {
            return;
          }
            
          if (sParam.host in connsPerServer) {
            connsPerServer [sParam.host] ++;
          }
          else {
            connsPerServer [sParam.host] = 1;
          }
          conns ++;
          
          sParam.startTime = now;
          sParam.status = UnMHTSave.STATUS_FILE_NET;
          
          let uri = null;
          let referrer = null;
          
          try {
            uri = Services.io.newURI (sParam.currentLocation, null, null);
          }
          catch (e) {
            sParam.status = UnMHTSave.STATUS_NG;
            return;
          }
          if (sParam.referrer) {
            try {
              referrer = Services.io.newURI (sParam.referrer, null, null);
            }
            catch (e) {
            }
          }
          sParam.loadContext
            = UnMHTNetFileLoader.createContext (uri, referrer, true, sParam);
          sParam.loadContext.start ();
        }
      }.bind (this));
    
    let na = this._saveParams.length - (ok + ng);
    if (na > 0) {
      this.progress = (ok + na_progress) / this._saveParams.length;
    }
    else {
      this.progress = 1;
    }
    this._listener.onProgress (this, currentFile, currentSize, ok, na, ng);
    
    if (na > 0) {
      UnMHTTimer.setTimeout
        (function () {
          this._checkProgress ();
        }.bind (this), 100);
    }
    else {
      UnMHTTimer.setTimeout
      (function () {
        this._storeToFile ();
      }.bind (this), 100);
    }
  },
  
  /**
   * mht で保存 - ファイルの保存
   */
  _storeToFile : function () {
    try {
    if (this._isCanceled) {
      UnMHTMaxRunTime.restoreTime ();
      this._isStopped = true;
      this._listener.onCancel (this);
      this.destruct ();
      return;
    }
    
    let fstream = UnMHTFileUtil.openOutputStream (this._targetFile);
    if (!fstream) {
      UnMHTMaxRunTime.restoreTime ();
      this._isStopped = true;
      this._listener.onError (this, UnMHTSave.ERROR_WRITE_FILE, null);
      this.destruct ();
      return;
    }
    
    let setPartData = function (part) {
      if (part.isMultipart) {
        part.parts = part.parts.filter (function (p) setPartData (p));
        return true;
      }
        
      let sParam = part.sParam;
      if (!sParam) {
        return true;
      }
      
      if (sParam.status == UnMHTSave.STATUS_NG
          || sParam.status == UnMHTSave.STATUS_IGNORE) {
        UnMHTMIMEUtil.destructPart (part);
        return false;
      }
        
      part.contentTransferEncoding
      = sParam.isText ? "quoted-printable" : "base64";
      part.charset = sParam.charset;
      part.mimetype = sParam.mimetype;
      
      let location = sParam.location;
      part.contentID = sParam.cid;
      
      let showContentID = false;
      if (UnMHTMailPrivacy.isMailURL (location)) {
        if (this._mailPart) {
          let re = location.match (/^.+\?part=([^&]+).*/);
          if (re) {
            let partID = re [1];
            let sourcePart
              = partID.split (/\./)
              .map (function (i) parseInt (i, 10))
              .slice (1)
              .reduce (function (p, i) {
                  if (!p) {
                    return null;
                  }
                  if (!p.parts) {
                    return null;
                  }
                
                  if (i - 1 >= p.parts.length) {
                    return null;
                  }
                  return p.parts [i - 1];
                }, this._mailPart);
            if (sourcePart
                && sourcePart.contentID) {
              showContentID = true;
              part.contentID = sourcePart.contentID;
            }
          }
        }
        
        location = UnMHTMailPrivacy.hideMailURL (location, this._location);
      }
      part.contentLocation = location;
      
      part.body = sParam.content;
      
      UnMHTMIMEUtil.addField (part, "Content-Type",
                              part.mimetype
                              + (part.charset ? 
                                 (";charset=\"" + part.charset + "\"")
                                 : ""));
      UnMHTMIMEUtil.addField (part, "Content-Transfer-Encoding",
                              part.contentTransferEncoding);
      
      if (!part.contentLocation
          || showContentID) {
        UnMHTMIMEUtil.addField (part, "Content-ID",
                                "<" + part.contentID + ">");
      }
      
      if (part.contentLocation) {
        UnMHTMIMEUtil.addField (part, "Content-Location",
                                part.contentLocation);
      }
        
      return true;
    }.bind (this);
    setPartData (this._topPart);
    
    if (this._startPart
        && this._startPart.mimetype) {
      this._topPart.startMimetype = this._startPart.mimetype;
      
      UnMHTMIMEUtil.addField (this._topPart, "Content-Type",
                              "multipart/related"
                              + ";boundary=\"" + this._topPart.boundary + "\""
                              + ";type=\"" + this._topPart.startMimetype + "\"");
    }
    else if (this._isMixed) {
      this._topPart.startMimetype = "";
      
      UnMHTMIMEUtil.addField (this._topPart, "Content-Type",
                              "multipart/mixed"
                              + ";boundary=\"" + this._topPart.boundary + "\"");
    }
    else if (this._isAlternative) {
      this._topPart.startMimetype = "";
      
      UnMHTMIMEUtil.addField (this._topPart, "Content-Type",
                              "multipart/alternative"
                              + ";boundary=\"" + this._topPart.boundary + "\"");
    }
    else {
      /* **** FIXME: この分岐は本当は来てはいけない **** */
      UnMHTMIMEUtil.addField (this._topPart, "Content-Type",
                              "multipart/related"
                              + ";boundary=\"" + this._topPart.boundary + "\"");
    }
    if (this._isTabs) {
      UnMHTMIMEUtil.addField (this._topPart, "X-UnMHT-tabs", "true");
    }
    
    UnMHTMIMEUtil.encodeMessage (this._topPart, fstream);
    
    fstream.close ();
        
    if (UnMHTPref.getPref ("save.history.register")) {
      this._registerHistory ();
    }
    
    UnMHTMaxRunTime.restoreTime ();
    this._isStopped = true;
    this.progress = 1;
    this._listener.onSave
    (this,
     this._saveParams
     .filter (function (sParam) sParam.status == UnMHTSave.STATUS_NG)
     .map (function (sParam) sParam.currentLocation));
    this.destruct ();
    }
    catch (e) {
      Cu.reportError (e + "\n" + e.stack);
    }
  },
  
  /**
   * ドキュメントをシリアライズする
   *
   * @param  UnMHTSaveParam sParam
   *         対象の保存情報
   * @return Boolean
   *         終了したか
   */
  _serializeDocument : function (sParam) {
    if (sParam.targetDocument.doctype) {
      sParam.content
      += "<!DOCTYPE "
      + sParam.targetDocument.doctype.name
      + ((sParam.targetDocument.doctype.publicId
          && sParam.targetDocument.doctype.systemId)
         ? (" PUBLIC \""
            + UnMHTHTMLUtil.escapeEntity
            (sParam.targetDocument.doctype.publicId)
            + "\" \""
            + UnMHTHTMLUtil.escapeEntity
            (sParam.targetDocument.doctype.systemId)
           + "\"")
         : "")
      + ">\n";
    }
    
    let tmpMeta = null;
    let tmpBaseElement = null;
    let tmpAttachmentBox = null;
    let nodes = sParam.targetDocument.getElementsByTagName ("head");
    if (nodes.length > 0) {
      let head = nodes [0];
      if (sParam.targetDocument.characterSet) {
        let hasCharsetMeta = false;
        let nodes = head.getElementsByTagName ("meta");
        hasCharsetMeta
        = Array.some
        (nodes,
         function (node)
         (node.hasAttribute ("http-equiv")
          && node.getAttribute ("http-equiv").match (/Content-Type/i)
          && node.hasAttribute ("content")
          && node.getAttribute ("content").match (/charset=/i))
         || node.hasAttribute ("charset"));
        
        if (!hasCharsetMeta) {
          tmpMeta = sParam.targetDocument.createElement ("meta");
          tmpMeta.setAttribute ("http-equiv", "Content-Type");
          tmpMeta.setAttribute
            ("content",
             "text/html; charset="
             + sParam.targetDocument.characterSet.toLowerCase ());
          head.insertBefore (tmpMeta, head.firstChild);
        }
      }
        
      if (!sParam.hasBaseElement) {
        tmpBaseElement = sParam.targetDocument.createElement ("base");
        tmpBaseElement.setAttribute ("href", sParam.baseDir);
        head.appendChild (tmpBaseElement);
      }
    }
    if (sParam.attachments) {
      tmpAttachmentBox = sParam.targetDocument.createElement ("div");
      let hr = sParam.targetDocument.createElement ("hr");
      tmpAttachmentBox.appendChild (hr);
      let ul = sParam.targetDocument.createElement ("ul");
      tmpAttachmentBox.appendChild (ul);
      sParam.attachments.forEach (function ([name, url]) {
          let li = sParam.targetDocument.createElement ("li");
          ul.appendChild (li);
          let a = sParam.targetDocument.createElement ("a");
          li.appendChild (a);
          a.setAttribute ("href", url);
          a.appendChild (sParam.targetDocument.createTextNode (name));
        });
      tmpBaseElement.setAttribute ("href", sParam.baseDir);
      sParam.targetDocument.body.appendChild (tmpAttachmentBox);
    }
    
    sParam.content
    += UnMHTDOMUtil.convertToPlainText
    (sParam.location, sParam.targetDocument.documentElement,
     sParam.targetSelection, {
      noScript : sParam.saveState == UnMHTSave.SAVESTATE_CURRENT_NOJS,
      isMail : sParam.isMail,
      saveCanvas : sParam.saveState == UnMHTSave.SAVESTATE_CURRENT_NOJS
    });
    
    if (tmpAttachmentBox) {
      tmpAttachmentBox.parentNode.removeChild (tmpAttachmentBox);
    }
    if (tmpBaseElement) {
      tmpBaseElement.parentNode.removeChild (tmpBaseElement);
    }
    if (tmpMeta) {
      tmpMeta.parentNode.removeChild (tmpMeta);
    }
    
    sParam.content
    = UnMHTUnicodeConverter.convertFromUnicodeWithEntity
    (sParam.content, sParam.charset);
    
    if (sParam.eFileInfo) {
      sParam.content
      = UnMHTExtractor.unmodifyAttribute (sParam.eFileInfo, sParam.content);
      sParam.content
      = UnMHTExtractor.unmodifyCSS (sParam.eFileInfo, sParam.content);
    }
    
    UnMHTSave.modifyContent (sParam, sParam.savedFrom, this._saveDate);
    
    return true;
  },
  
  /**
   * ファイルを収集する
   *
   * @param  UnMHTSaveParam sParam
   *         対象の保存情報
   * @return Boolean
   *         終了したか
   */
  _gatherDocument : function (sParam) {
    let i = 0;
    let startTime = (new Date ()).getTime ();
    for (;;) {
      let node = sParam.treeScanner.getNext ();
      if (!node) {
        break;
      }
      
      if (node instanceof Ci.nsIDOMElement) {
        this._gatherFilesFromNode
          (sParam.targetDocument, sParam.location, sParam.saveState, node,
           sParam.baseDir, sParam.depth,
           sParam.treeScanner.isHead ? null : sParam.targetSelection,
           sParam.referrer, sParam.isMail);
      }
      
      i ++;
      if (i == 1000) {
        i = 0;
        let now = (new Date ()).getTime ();
        if (now > startTime + 100) {
          return false;
        }
      }
    }
    
    this._gatherFonts (sParam);
    
    return true;
  },
  
  /**
   * フォントを収集する
   *
   * @param  UnMHTSaveParam sParam
   *         対象の保存情報
   */
  _gatherFonts : function (sParam) {
    try {
      let domUtil
      = Cc ["@mozilla.org/inspector/dom-utils;1"].getService (Ci.inIDOMUtils);
      let range = sParam.targetDocument.createRange ();
      range.selectNodeContents (sParam.targetDocument.documentElement);
      let fontList = domUtil.getUsedFontFaces (range);
      for (let i = 0, length = fontList.length; i < length; i ++) {
        let font = fontList.item (i);
        if (!font.URI) {
          continue;
        }
        this._checkAndPush (font.URI, Ci.nsIContentPolicy.TYPE_FONT,
                            sParam.targetDocument, 0, sParam.referrer);
      }
    }
    catch (e) {
    }
  },
  
  /**
   * フィールドを追加する
   *
   * @param  Array fieldList
   *         ヘッダ
   *           [[String フィールド名, String フィールド値], ...]
   */
  _addFieldList : function (fieldList) {
    fieldList.forEach (function ([name, value]) {
        this.addField (name, value);
      }.bind (this));
  },
  
  /**
   * ダウンロード履歴に記録する
   */
  _registerHistory : function () {
    let mimeInfo
    = Cc ["@mozilla.org/mime;1"].getService (Ci.nsIMIMEService)
    .getFromTypeAndExtension ("message/rfc822", "mht");
    
    let fileSize = this._targetFile.fileSize;
    let listener = {
      dl : null,
      QueryInterface : XPCOMUtils.generateQI ([Ci.nsIStreamListener,
                                               Ci.nsIRequestObserver,
                                               Ci.nsICancelable]),
      onStopRequest : function (request, context, statusCode) {
        this.dl.onProgressChange
        (null, request, fileSize, fileSize, fileSize, fileSize);
        
        this.dl.onStateChange
        (null, request,
         Ci.nsIWebProgressListener.STATE_STOP
         | Ci.nsIWebProgressListener.STATE_IS_NETWORK,
         Cr.NS_OK);
        
        this.dl = null;
      },
        
      cancel : function () {},
      onStartRequest : function () {},
      onDataAvailable : function () {}
    };
    
    let source = Services.io.newURI (this._location, null, null);
    let target = Services.io.newFileURI (this._targetFile);
    let displayName = this._targetFile.leafName;
    let startTime = null;
    let tempFile = null;
    let isPrivate = false;
    
    try {
      listener.dl
        = Services.downloads.addDownload
        (Ci.nsIDownloadManager.DOWNLOAD_TYPE_DOWNLOAD,
         source, target, displayName, mimeInfo,
         startTime, tempFile, listener, isPrivate);
    }
    catch (e) {
      listener.dl
        = Services.downloads.addDownload
        (Ci.nsIDownloadManager.DOWNLOAD_TYPE_DOWNLOAD,
         source, target, displayName, mimeInfo,
         startTime, tempFile, listener);
    }
    
    let channel
    = Services.io.newChannelFromURI
    (Services.io.newURI ("data:text/plain,", null, null));
    
    channel.asyncOpen (listener, null);
  },
  
  /**
   * トップレベルのパートを作成する
   *
   * @param  String subject
   *         ファイルの subject
   */
  _prepareTopPart : function (subject) {
    let part = new UnMHTMIMEPart ();
    part.isMultipart = true;
    part.boundary = UnMHTMIMEUtil.createUniqueBoundary ();
    UnMHTMIMEUtil.addField (part, "From",
                            "<Saved by UnMHT Ver." + UnMHTVersion + ">");
    UnMHTMIMEUtil.addField (part, "Subject", subject);
    UnMHTMIMEUtil.addField (part, "Date",
                            UnMHTMIMEUtil.formatDate (this._saveDate));
    UnMHTMIMEUtil.addField (part, "MIME-Version", "1.0");
    part.parts = [];
    
    this._topPart = part;
  }
};

/**
 * タブを MHT で保存
 */
let UnMHTSave = {
  /* ファイルの取得状況 */
  /* 元のファイル (通常のファイル) */
  STATUS_FILE_NA         : 0x0001, /* Number  未取得 */
  STATUS_FILE_NET        : 0x0011, /* Number  ネットワーク開始 */
  
  /* 現在の状態 (ドキュメント) */
  STATUS_DOC_CUR_NA      : 0x0101, /* Number  未取得 */
  STATUS_DOC_CUR_SERIAL  : 0x0111, /* Number  シリアライズ中 */
  STATUS_DOC_CUR_GATHER  : 0x0121, /* Number  ファイル回収中 */
  
  /* 元のファイル (ドキュメント) */
  STATUS_DOC_ORI_NA      : 0x0201, /* Number  未取得 */
  STATUS_DOC_ORI_GATHER  : 0x0211, /* Number  ファイル回収中
                                    *         ここから先は STATUS_FILE_NA */
  
  /* メールデータ取得中 */
  STATUS_MAIL_CUR_MAIL   : 0x0301, /* Number  現在の状態 - メール取得中 */
  STATUS_MAIL_ORI_MAIL   : 0x0311, /* Number  元のファイル - メール取得中 */
  STATUS_MAIL_ORI_NA     : 0x0312, /* Number  元のファイル - ファイル未回収 */
  STATUS_MAIL_ORI_GATHER : 0x0313, /* Number  元のファイル - ファイル回収中 */
  
  STATUS_OK              : 0x1001, /* Number  取得完了 */
  STATUS_NG              : 0x1002, /* Number  取得失敗 */
  STATUS_IGNORE          : 0x1003, /* Number  このパートを無視する */
  
  /* エラー定数 */
  ERROR_WRITE_FILE       : 0x0001, /* Number  ファイル書き込み失敗 */
  ERROR_ADD_DOCUMENT     : 0x0002, /* Number  ドキュメント追加失敗 */
  
  /* 保存状態のマスク */
  SAVESTATE_MASK_ALL       : 0xffff, /* Number  全て */
  SAVESTATE_MASK_ORIGINAL  : 0x000f, /* Number  元のファイル */
  SAVESTATE_MASK_CURRENT   : 0x00f0, /* Number  現在の状態 */

  /* 保存状態 */
  SAVESTATE_ORIGINAL       : 0x0001, /* Number  元のファイル */
  SAVESTATE_CURRENT        : 0x0010, /* Number  現在の状態 */
  SAVESTATE_CURRENT_NOJS   : 0x0020, /* Number  現在の状態 (スクリプト削除) */
  SAVESTATE_UNKNOWN        : 0x0000, /* Number  不明 */

  saveStateList : [],   /* Array 保存状態の数値表現と文字列表現のリスト
                         *   [[Number 数値表現, String 文字列表現], ...] */
  
  MAX_IMPORT_DEPTH : 10, /* Number  最大の import の深さ */
  
  CACHE_TIMEOUT : 5,     /* Number  キャッシュ取得のタイムアウト */
  RECEIVE_TIMEOUT : 15,  /* Number  ネットワーク取得のタイムアウト */
  
  /**
   * 保存コンテキストを作成する
   *
   * @param  nsIDOMWindow window
   *         実行中のウィンドウ
   * @param  nsIFile targetFile
   *         対象のファイル,
   * @param  String location
   *         メールを保存する際のルートドキュメントの URI
   * @param  String subject
   *         ファイルの subject
   * @param  IUnMHTSaveListener listener
   *         イベントリスナ
   * @return UnMHTSaveContext
   *         保存コンテキスト
   */
  createContext : function (window, targetFile, location, subject, listener) {
    return new UnMHTSaveContext
    (window, targetFile, location, subject, listener);
  },
  
  /**
   * 保存可能かどうかを調べる
   *
   * @param  nsIDOMWindow window
   *         実行中のウィンドウ
   *         null 可
   * @param  nsIDOMDocument targetDocument
   *         対象のドキュメント
   * @return Boolean
   *         保存可能かどうか
   */
  isSavable : function (window, targetDocument) {
    try {
      /* ==== FIXME: 最近動作チェックしてない ==== */
      if (window
          && "gIeTab" in window
          && window.gIeTab.isIeEngine ()) {
        return false;
      }
    }
    catch (e) {
    }
    
    if (window
        && UnMHTWindowUtil.isMailWindow (window)
        && targetDocument.location.href == "about:blank") {
      /* 複数のメッセージを選択しているとき等 */
      return false;
    }
    if (window
        && UnMHTWindowUtil.isMessagePaneCollapsed (window)) {
      return false;
    }
    
    if ("contentType" in targetDocument
        && targetDocument.contentType
        && (targetDocument.contentType.match (/^text/)
            || targetDocument.contentType.match (/\/x(ht)?ml(\+xml)?$/))) {
      return true;
    }
    
    return false;
  },
  
  /**
   * URI がロード可能かどうか調べる
   *
   * @param  Number type
   *         nsIContentPolicy の対象の種類
   * @param  String location
   *         対象 URL
   * @param  nsIDOMElement context
   *         読み込み元の要素
   *         読み込み元が無い場合 null 可能
   * @return Boolean
   *         ロード可能か
   */
  checkPolicy : function (type, location, context) {
    let uri;
    try {
      uri = Services.io.newURI (location, null, null);
    }
    catch (e) {
      return false;
    }
    let originURI = null;
    if (context) {
      let doc
        = (context instanceof Ci.nsIDOMDocument)
        ? context
        : context.ownerDocument;
      if (doc) {
        try {
          originURI = Services.io.newURI (doc.URL, null, null);
        }
        catch (e) {
        }
      }
    }
    
    try {
      let result
      = this._contentPolicy.shouldLoad
      (type, uri, originURI, context, null, null);
      return (result == Ci.nsIContentPolicy.ACCEPT);
    }
    catch (e) {
    }
    
    return true;
  },
  
  /**
   * URL をコメントとして挿入する
   *
   * @param  String content
   *         ファイルの内容
   * @param  String location
   *         ドキュメントの URL
   * @return String
   *         ファイルの内容
   */
  insertComment : function (content, location) {
    if (UnMHTScheme.isUnMHTURI (location)) {
      location = UnMHTScheme.deUnMHTURI (location);
    }
    
    let comment = "<!-- Saved from " + location + " -->";
    if (content.match (/<html[^>]*>/i)) {
      content
        = content.replace
        (/<html[^>]*>/i,
         function (matched) {
          return matched + comment;
        });
    }
    else if (content.match (/<head[^>]*>/i)) {
      content
      = content.replace
      (/<head[^>]*>/i,
       function (matched) {
        return matched + comment;
      });
    }
    else if (content.match (/<body[^>]*>/i)) {
      content
      = content.replace
      (/<body[^>]*>/i,
       function (matched) {
        return matched + comment;
      });
    }
    else {
      content = comment + content;
    }
    
    return content;
  },

  /**
   * URL をリンクとして挿入する
   *
   * @param  String content
   *         ファイルの内容
   * @param  String location
   *         ドキュメントの URL
   * @param  Date saveDate
   *         保存日時
   * @return String
   *         ファイルの内容
   */
  insertLink : function (content, location, saveDate) {
    if (UnMHTScheme.isUnMHTURI (location)) {
      location = UnMHTScheme.deUnMHTURI (location);
    }
    
    let divStyles = [];
    if (this._insertLinkPos == "bottom") {
      divStyles.push ("clear: both");
      divStyles.push ("margin-top: 2em");
    }
    divStyles.push ("font-size: 0.8em");
    divStyles.push ("border: 1px solid black");
    divStyles.push ("background-color: white");
    divStyles.push ("color: black");
    divStyles.push ("padding: 0.5em");
    divStyles.push ("text-align: center");
    
    let aStyles = [];
    aStyles.push ("color: blue");
    aStyles.push ("background-color: inherit");
    aStyles.push ("text-decoration: underline");
    
    let link
    = "<div style=\"" + divStyles.join (";") + ";\">"
    + "Saved from <a class=\"unmht_link_to_original\" style=\""
    + aStyles.join (";") + "\" href=\""
    + UnMHTHTMLUtil.escapeEntity (location) + "\">"
    + UnMHTHTMLUtil.escapeEntity (location)
    + "</a>"
    + (this._enableInsertLinkDate
       ? ("<br />" + UnMHTHTMLUtil.escapeEntity (saveDate.toString ()))
       : "")
    + (this._enableInsertLinkVersion
       ? ("<br />" + "Saved by UnMHT Ver." + UnMHTVersion)
       : "")
    + "</div>";
    
    if (this._insertLinkPos == "bottom") {
      if (content.match (/<\/body>/i)) {
        content
          = content.replace
          (/<\/body>/i,
           function (matched) {
            return link + matched;
          });
      }
    }
    else {
      if (content.match (/<body[^>]*/i)) {
        content
        = content
        .replace
        (/<body[^>]*>/i,
         function (matched) {
          return matched + link + "<div style=\"position: relative\">";
        })
        .replace
        (/<\/body>/i,
         function (matched) {
          return "</div>" + matched;
        });
      }
    }
    
    return content;
  },
  
  /**
   * ファイルを修正する
   *
   * @param  UnMHTSaveParam param
   *         保存情報
   * @param  String location
   *         対象のファイルの URI
   * @param  Date saveDate
   *         保存日時
   */
  modifyContent : function (sParam, location, saveDate) {
    if (sParam.isHTML && !sParam.isIFrame) {
      if (this._enableInsertURL) {
        sParam.content = this.insertComment (sParam.content, location);
      }
      if (this._enableInsertLink) {
        if (!sParam.isLocal || !this._enableInsertLinkNoLocal) {
          sParam.content
          = this.insertLink (sParam.content, location, saveDate);
        }
      }
    }
  },
  
  /**
   * CSS の階層が深い場合の警告を表示する
   *
   * @param  nsIDOMWindow window
   *         実行中のウィンドウ
   * @return String
   *         今回の動作
   */
  showDeepCSSConfirm : function (window) {
    let confirmURL = "chrome://unmht/content/confirm.xul";
    let features = "chrome,dialog,modal,centerscreen";
    let result = {
      result : 0,
      applyToAll: false,
      filename : ""
    };
    let arg = {
      result   : result,
      title    : UnMHTStringBundle.get ("deep_css_title"),
      message1 : UnMHTStringBundle.get ("deep_css_message"),
      message2 : UnMHTStringBundle.get ("deep_css_message2"),
      message3 : UnMHTStringBundle.get ("deep_css_message3"),
      accept   : UnMHTStringBundle.get ("deep_css_save"),
      cancel   : UnMHTStringBundle.get ("deep_css_ignore"),
      toall    : UnMHTStringBundle.get ("deep_css_do_not_ask_again")
    };
    window.openDialog (confirmURL, "", features, arg);
    
    let action;
    
    if (result.result == "yes") {
      action = "save";
    }
    else if (result.result == "no") {
      action = "ignore";
    }
    
    if (result.applyToAll) {
      UnMHTPref.setPref ("save.deep_import.action", action);
    }
    
    return action;
  },

  /**
   * 内容を保存するかどうかの警告を表示する
   *
   * @param  nsIDOMWindow window
   *         実行中のウィンドウ
   * @param  Number type
   *         内容の種類
   *           UnMHTSave.SAVE_CONTENTS_*
   * @return String
   *         今回の動作
   */
  showSaveContentsConfirm : function (window, type) {
    let confirmURL = "chrome://unmht/content/confirm.xul";
    let features = "chrome,dialog,modal,centerscreen";
    let result = {
      result : 0,
      applyToAll: false,
      filename : ""
    };
    let arg = {
      result   : result,
      title    : UnMHTStringBundle.get ("save_contents_" + type + "_title"),
      message1 : UnMHTStringBundle.get ("save_contents_" + type + "_message"),
      message2 : UnMHTStringBundle.get ("save_contents_" + type + "_message2"),
      accept   : UnMHTStringBundle.get ("save_contents_save"),
      cancel   : UnMHTStringBundle.get ("save_contents_ignore"),
      toall    : UnMHTStringBundle.get ("save_contents_do_not_ask_again")
    };
    window.openDialog (confirmURL, "", features, arg);
    
    let action;
    
    if (result.result == "yes") {
      action = "save";
    }
    else if (result.result == "no") {
      action = "ignore";
    }
    
    if (result.applyToAll) {
      UnMHTPref.setPref ("save.contents." + type, action);
    }
    
    return action;
  },
  
  /**
   * 同名のファイルが存在した場合の警告を表示する
   *
   * @param  nsIDOMWindow window
   *         実行中のウィンドウ
   * @param  nsIFile targetFile
   *         同名のファイル
   * @param  String filenameBase
   *         ファイル名の拡張子以外
   * @param  String showApplyToAll
   *         全てに適用チェックを表示するか
   * @return String
   *         今回の動作
   */
  showOverwriteConfirm : function (window, targetFile, filenameBase,
                                   showApplyToAll) {
    let result = {
      result : 0,
      applyToAll: false,
      filename : ""
    };
    
    let baseDir = targetFile.parent.clone ();
    
    let confirmURL = "chrome://unmht/content/confirm.xul";
    let features = "chrome,dialog,modal,centerscreen";
    let arg = {
      result   : result,
      title    : UnMHTStringBundle.get ("overwrite_title"),
      message1 : (UnMHTStringBundle.get ("overwrite_pre")
                  + " " + targetFile.leafName + " "
                  + UnMHTStringBundle.get ("overwrite_post")),
      message2 : (UnMHTStringBundle.get ("overwrite_lastaccess")
                  + " "
                  + UnMHTFormatter.getReadableTime
                  (targetFile.lastModifiedTime)),
      message3 : UnMHTStringBundle.get ("overwrite_post2"),
      accept   : UnMHTStringBundle.get ("replace"),
      extra2   : UnMHTStringBundle.get ("abort"),
      rename   : UnMHTStringBundle.get ("rename"),
      toall    : null,
      filename : UnMHTSave.getUniqueFilename (baseDir, filenameBase),
      baseDir  : baseDir,
      exist1   : UnMHTStringBundle.get ("overwrite_pre"),
      exist2   : UnMHTStringBundle.get ("overwrite_post")
    };
    if (showApplyToAll) {
      arg.toall = UnMHTStringBundle.get ("overwrite_all");
    }
    window.openDialog (confirmURL, "", features, arg);
    
    return result;
  },
  
  /**
   * ユニークなファイル名を返す
   *
   * @param  nsIFile baseDir
   *         親ディレクトリ
   * @param  String filenameBase
   *         ファイル名の拡張子以外
   * @return String
   *         ユニークなファイル名
   */
  getUniqueFilename : function (baseDir, filenameBase) {
    let dupInit = UnMHTPref.getPref ("save.filename.dup.rename.init");
    let dupDigits = UnMHTPref.getPref ("save.filename.dup.rename.digits");
    
    let padDup = function (n, dupDigits) {
      let s = n + "";
      while (s.length < dupDigits) {
        s = "0" + s;
      }
      return s;
    };
    
    let i = dupInit;
    let filename = filenameBase + ".mht";
    let file = baseDir.clone ();
    file.append (filename);
    while (file.exists ()) {
      filename = filenameBase + "-" + padDup (i, dupDigits) + ".mht";
      file = baseDir.clone ();
      file.append (filename);
      i ++;
    }
    
    return filename;
  },
  
  /**
   * 保存する際のファイル名を取得する
   *
   * @param  nsIDOMDocument targetDocument
   *         対象のドキュメント
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   * @param  String saveFilename
   *         ファイル名の種類
   *           title/leafname/custom
   * @param  Date date
   *         日時
   *         カスタムファイル名に使用する
   */
  getFilename : function (targetDocument, scrapBookData, saveFilename, date) {
    let [uri, title]
    = this.getLocationAndTitle (targetDocument, scrapBookData);
    uri = decodeURI (uri);
    title = title || uri;
    
    let localFile = {value : null};
    if (UnMHTPref.getPref ("save.local.same_dir")
        && this.isLocalFile (uri, localFile)) {
      return localFile.value.leafName.replace (/\.[^\.]+$/, "")
        + UnMHTPref.getPref ("save.local.same_dir.append");
    }
    
    let filename = "";
    
    let re = uri.match (/[^\?]+\/([^\/\?]+)(\?.+)?(#.+)?$/);
    if (re) {
      filename = re [1].replace (/\.[^\.]+$/, "");
    }
    else {
      filename = "index";
    }
    
    if (!saveFilename) {
      saveFilename = UnMHTPref.getPref ("save.filename");
    }
    if (UnMHTMailPrivacy.isMailURL (uri)
        && saveFilename == "filename") {
      saveFilename = "title";
    }
    
    switch (saveFilename) {
      case "title": {
        filename = UnMHTFileUtil.validateFileName (title) || filename;
        break;
      }
      case "custom": {
        filename
        = UnMHTCustomFilename.format (title, filename, uri, date);
        break;
      }
      default: {
        filename = UnMHTFileUtil.validateFileName (filename);
      }
    }
    
    return filename;
  },
  
  /**
   * ローカルファイルかどうかを返す
   * ローカルファイルであればローカルファイルを格納する
   *
   * @param  String location
   *         対象の URI
   * @param  Object
   *         {
   *           value: nsIFile  ローカルファイル
   *         }
   * @return Boolean
   *         ローカルファイルかどうか
   */
  isLocalFile : function (location, file) {
    if (UnMHTScheme.isUnMHTURI (location)) {
      location = UnMHTScheme.deUnMHTURI (location);
    }
    
    if (!location.match (/^file:\/\/\//)) {
      return false;
    }
    
    file.value = UnMHTFileUtil.getFileFromURLSpec (location);
    
    return true;
  },
  
  /**
   * ドキュメントのアドレスとタイトルを取得する
   *
   * @param  nsIDOMDocument targetDocument
   *         対象のドキュメント
   * @param  Object scrapBookData
   *         ScrapBook のデータ
   * @retrun Array
   *           [Stirng アドレス, String タイトル]
   */
  getLocationAndTitle : function (targetDocument, scrapBookData) {
    if (scrapBookData) {
      return [scrapBookData ["source"], scrapBookData ["title"]];
    }
    else {
      return [targetDocument.location.href, targetDocument.title];
    }
  },
  
  /**
   * 保存状態を文字列表現から数値表現に変換する
   *
   * @param  String _str
   *         文字列表現
   * @return Number
   *         数値表現
   */
  stringToSaveState : function (_str) {
    return UnMHTArrayUtil.find
    (this.saveStateList,
     function ([state, str]) str == _str,
     [this.SAVESTATE_UNKNOWN, "unknown"]) [0];
  },
  
  /**
   * 保存状態を数値表現から文字列表現に変換する
   *
   * @param  Number _state
   *         数値表現
   * @return String
   *         文字列表現
   */
  saveStateToString : function (_state) {
    return UnMHTArrayUtil.find
    (this.saveStateList,
     function ([state, str]) state == _state,
     [this.SAVESTATE_UNKNOWN, "unknown"]) [1];
  },
  
  /**
   * マスクでフィルタした保存状態の数値表現のリストを返す
   *
   * @param  Number mask
   *         保存状態のマスク
   * @return Array
   *         保存状態の数値表現のリスト
   */
  getSaveStateList : function (mask) {
    return this.saveStateList
    .filter (function ([state, str]) state & mask)
    .map (function ([state, str]) state);
  },
  
  /**
   * 保存状態の数値表現と追加ファイルの有無から
   * ファイルタイプの文字列を作成する
   *
   * @param  Number saveState
   *         保存状態の数値表現
   * @param  Boolean useAdd 
   *         追加ファイルの有無
   * @return String
   *         ファイルタイプ
   */
  makeFiletype : function (saveState, useAdd) {
    let filetype = UnMHTStringBundle.get ("filetype");
    
    switch (saveState) {
      case UnMHTSave.SAVESTATE_CURRENT: {
        filetype += " " + UnMHTStringBundle.get ("filetype_current");
        break;
      }
      case UnMHTSave.SAVESTATE_CURRENT_NOJS: {
        filetype += " " + UnMHTStringBundle.get ("filetype_current")
          + " (" + UnMHTStringBundle.get ("filetype_nojs") + ")";
        break;
      }
      case UnMHTSave.SAVESTATE_ORIGINAL: {
        filetype += " " + UnMHTStringBundle.get ("filetype_original");
        break;
      }
    }
    
    if (useAdd) {
      filetype += " + " + UnMHTStringBundle.get ("filetype_add");
    }
    
    return filetype;
  },
  
  /**
   * ドキュメントが生きているかどうか調べる
   *
   * @param  nsIDOMDocument targetDocument
   *         対象のドキュメント
   * @return Boolean
   *         ドキュメントが生きているか
   */
  isAlive : function (targetDocument) {
    try {
      return targetDocument.location.href != "";
    }
    catch (e) {
      return false;
    }
  },
  
  /**
   * 追加 URI を解析する
   *
   * @param  String text
   *         行ごとに URI を記述したテキスト
   * @return Array
   *         URI リスト
   *           [String URI, ...]
   */
  parseAddURIs : function (text) {
    return text.split (/[\r\n]/)
    .map (function (line) line.replace (/^\s*|\s*$/g, ""))
    .filter (function (uri) uri)
  },
  
  /* ==== private ==== */
  
  _enableInsertURL : false,         /* Boolean  URL をコメントとして挿入 */
  _enableInsertLink : false,        /* Boolean  URL をリンクとして挿入 */
  _insertLinkPos : "bottom",        /* String  ページ先頭/ページ末尾 */
  _enableInsertLinkDate : false,    /* Boolean  保存日時 */
  _enableInsertLinkVersion : false, /* Boolean  バージョン情報 */
  _enableInsertLinkNoLocal : false, /* Boolean  ローカルファイルには
                                     *          適用しない */
  
  _contentPolicy : null, /* nsIContentPolicy  ブロックチェック用 */
  
  /**
   * 初期化処理
   */
  init : function () {
    this._contentPolicy
    = Cc ["@mozilla.org/layout/content-policy;1"]
    .getService (Ci.nsIContentPolicy);
    
    this.saveStateList = [
      [this.SAVESTATE_CURRENT,      "current"],
      [this.SAVESTATE_CURRENT_NOJS, "curnojs"],
      [this.SAVESTATE_ORIGINAL,     "original"]
      ];
    
    let f = function () {
      this._enableInsertLink = UnMHTPref.getPref ("save.insert.link");
      this._insertLinkPos = UnMHTPref.getPref ("save.insert.link.pos");
      this._enableInsertLinkDate = UnMHTPref.getPref ("save.insert.link.date");
      this._enableInsertLinkVersion
      = UnMHTPref.getPref ("save.insert.link.version");
      this._enableInsertLinkNoLocal
      = UnMHTPref.getPref ("save.insert.link.nolocal");
    
      this._enableInsertURL = UnMHTPref.getPref ("save.insert.url");
    }.bind (this);
    UnMHTPref.addListener (UnMHTPref.prefix + "save.insert", f, this);
    f ();
  },
  
  /**
   * 終了処理
   */
  term : function () {
    UnMHTPref.removeAllListeners (this);
  }
};
UnMHTSave.init ();
UnMHTShutdown.addListener (function (data, reason) {
    UnMHTSave.term ();
  });
