/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTDialog.jsm");
XPCOMUtils.importRelative (this, "UnMHTPref.jsm");
XPCOMUtils.importRelative (this, "UnMHTPrototype.jsm");
XPCOMUtils.importRelative (this, "UnMHTSave.jsm");
XPCOMUtils.importRelative (this, "UnMHTTimer.jsm");

let EXPORTED_SYMBOLS = ["UnMHTSaveOptionsDialogClass"];

/**
 * 保存オプション
 *
 * ウィンドウの引数
 * {
 *   showFilename  : Boolean  ファイル名を表示し、修正可能にするか (必須),
 *   filename      : String  ファイル名 (必須),
 *
 *   saveStateMask : Number  リスト表示する保存状態 (必須),
 *   saveState     : Number  デフォルトの保存状態 (必須),
 *   saveStatePref : String  選択された保存状態を格納する設定名 (必須),
 *
 *   callback      : Function  終了時コールバック (オプショナル)
 * }
 *
 * @param  nsIDOMWindow window
 *         対象のウィンドウ
 * @inherits   UnMHTDialog
 * @implements IUnMHTDialog
 */
function UnMHTSaveOptionsDialogClass (window) {
  UnMHTDialog.call (this, window);
}
UnMHTSaveOptionsDialogClass.prototype = {
  /* ==== IUnMHTDialog インターフェース ==== */
  
  /**
   * ロードイベント
   */
  onLoad : function () {
    let arg = this._window.arguments [0];
    
    this._showFilename = arg.showFilename;
    this._filename = arg.filename;
    if (this._showFilename) {
      this.el ("filename").value = this._filename;
      this.el ("filename_box").hidden = false;
    }
    
    UnMHTSave.getSaveStateList (arg.saveStateMask)
    .map (function (state) UnMHTSave.saveStateToString (state))
    .forEach (function (stateStr) {
        this.el ("savestate_" + stateStr).hidden = false;
      }.bind (this));
    this.el ("savestate").value
    = UnMHTSave.saveStateToString (arg.saveState);
    
    this._saveStatePref = arg.saveStatePref;
    
    if ("callback" in arg) {
      this._callback = arg.callback;
    }
    
    this.el ("save_insertlink").checked
    = UnMHTPref.getPref ("save.insert.link");
  },
  
  /**
   * ダイアログを OK したイベント
   *
   * @return Boolean
   *         ダイアログを閉じるか
   */
  onDialogAccept : function () {
    UnMHTPref.setPref (this._saveStatePref, this.el ("savestate").value);
    UnMHTPref.setPref ("save.insert.link", this.el ("save_insertlink").checked);
    
    let saveParam = {
      targetFilename : (this._showFilename
                        ? this.el ("filename").value
                        : this._filename),
      saveState : UnMHTSave.stringToSaveState (this.el ("savestate").value),
      addURIs : (UnMHTSave.parseAddURIs (this.el ("add_uris").value))
    };
    
    if (this._callback) {
      let callback = this._callback;
      UnMHTTimer.setTimeout
        (function () {
          callback (saveParam);
        }.bind (this), 100);
    }
    
    return true;
  },
  
  /* ==== private ==== */
  
  _showFilename : false, /* Boolean  名を表示し、修正可能にするか */
  _filename : "",        /* String  ファイル名 */
  
  _saveStatePref : "",   /* String 選択された保存状態を格納する設定名 */
  
  _callback : null       /* Function  終了時コールバック */
};
UnMHTPrototype.createInheritance (UnMHTSaveOptionsDialogClass, UnMHTDialog);
