/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;

let EXPORTED_SYMBOLS = ["UnMHTScheme"];

/**
 * URI の情報
 */
function UnMHTURIParam () {
}
UnMHTURIParam.prototype = {
  original: "",   /* String  本来の URI */
  scheme : "",    /* String  本来の URI の scheme */
  host : "",      /* String  本来の URI のホスト */
  path : "",      /* String  本来の URI のパス */
  fragment : "",  /* String  本来の URI のフラグメント */
  
  refName : "",   /* String  参照名 */
  baseURI : ""    /* String  開始パートの URI */
};
/**
 * unmht: スキーム操作
 */
let UnMHTScheme = {
  /**
   * unmht: スキームの URI にする
   *
   * @param  String uri
   *         URI
   * @return String
   *         unmht: スキームの URI
   */
  enUnMHTURI : function (uri) {
    let re = uri.match (/^([A-Za-z0-9\-]+):(\/\/)?([^\/]*)(\/\/)?(\/)?(.*)$/);
    if (re) {
      let [, scheme, sep1, host, sep2, sep3, path] = re;
      
      let sep = 0;
      if (sep1) {
        sep |= 1;
      }
      if (sep2) {
        sep |= 2;
      }
      if (sep3) {
        sep |= 4;
      }
      
      if (scheme.match (/^data$/i)
        && sep == 4) {
        path = host + "/" + path;
        host = "";
        sep = 0;
      }
            
      uri
        = "unmht://" + host + ".unmht" + "/"
        + scheme + "." + sep + "/" + path + "/";
    }
    
    return uri;
  },
    
  /**
   * unmht: スキームの URI かどうかを返す
   *
   * @param  String uri
   *         URI
   * @return Boolean
   *         unmht: スキームの URI かどうか
   */
  isUnMHTURI : function (uri) {
    let param = UnMHTScheme.getUnMHTURIParam (uri);
    if ("original" in param
        && param.original) {
      return true;
    }
        
    return false;
  },
    
  /**
   * unmht: スキームの URI を元に戻す
   *
   * @param  String uri
   *         unmht: スキームの URI
   * @return String
   *         URI
   */
  deUnMHTURI : function (uri) {
    let param = UnMHTScheme.getUnMHTURIParam (uri);
    if ("original" in param
        && param.original) {
      return param.original;
    }
        
    return uri;
  },
    
  /**
   * unmht: スキームの URI の情報を取得する
   *
   * @param  String uri
   *         unmht: スキームの URI
   * @return UnMHTURIParam
   *         URI の情報
   */
  getUnMHTURIParam : function (uri) {
    let param = new UnMHTURIParam ();
    
    let re = uri.match
    (/^unmht:\/\/([^\/]*)\.unmht\/([A-Za-z0-9\-]+)\.([0-9]+)\/(.*)\/([^\/#]*)(?:#(.*))?$/);
    if (!re) {
      return param;
    }
    
    let idn
    = Cc ["@mozilla.org/network/idn-service;1"]
    .getService (Ci.nsIIDNService);
      
    param.host     = idn.convertUTF8toACE (re [1]);
    param.scheme   = re [2].toLowerCase ();
    let sep        = parseInt (re [3], 10);
    param.path     = re [4];
    param.refName  = re [5];
    param.fragment = re [6];
      
    let sep1 = (sep & 1) ? "//" : "";
    let sep2 = (sep & 2) ? "//" : "";
    let sep3 = (sep & 4) ? "/" : "";
            
    param.original
    = param.scheme + ":" + sep1 + param.host + sep2 + sep3
    + param.path;
      
    param.baseURI
    = "unmht://" + param.host + ".unmht" + "/"
    + param.scheme + "." + sep + "/" + param.path + "/";
    
    return param;
  },
  
  /**
   * エラーページの URI を返す
   *
   * @param  String uri
   *         URI
   * @return String
   *         エラーページの URI
   */
  enErrorURI : function (uri) {
    uri = this.enUnMHTURI (uri);
    uri = uri.replace (/^unmht:/, "error:");
    uri = this.enUnMHTURI (uri);
    
    return uri;
  },
  
  /**
   * エラーページの URI を元に戻す
   *
   * @param  String uri
   *         エラーページの URI
   * @return String
   *         URI
   */
  deErrorURI : function (uri) {
    uri = this.deUnMHTURI (uri);
    uri = uri.replace (/^error:/, "unmht:");
    uri = this.deUnMHTURI (uri);
    
    return uri;
  },
  
  /**
   * 空のページの URI を返す
   *
   * @return String
   *         空のページの URI
   */
  createEmptyURI : function () {
    return this.enUnMHTURI ("empty:");
  },
};
