/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/Services.jsm");
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTFileLoader.jsm");
XPCOMUtils.importRelative (this, "UnMHTScheme.jsm");
XPCOMUtils.importRelative (this, "UnMHTTimer.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTStartupExtract"];

/**
 * アプリケーション起動以外のスタートアップ時に
 * タブにある MHT ファイルを展開する
 *
 * @implements IUnMHTStartupListener
 */
let UnMHTStartupExtract = {
  /* ==== IUnMHTStartupListener インターフェース ==== */
  
  /**
   * スタートアップ時のイベント
   *
   * @param  Object data
   *         Bootstrap データ
   * @param  Number reason
   *         理由
   */
  onStartup : function (data, reason) {
    if (reason == 0) {
      return;
    }
    
    UnMHTTimer.setTimeout (function () {
        this._reloadTabs ();
      }.bind (this), 100);
  },
  
  /* ==== private ==== */
  
  /**
   * タブにある MHT ファイルを展開する
   */
  _reloadTabs : function () {
    for (let window in XPCOMUtils.IterSimpleEnumerator
           (Services.wm.getEnumerator ("navigator:browser"), Ci.nsIDOMWindow)) {
      UnMHTWindowUtil.getDocuments (window).forEach
      (function (targetDocument) {
        let location = targetDocument.location.href;
        if (!UnMHTScheme.isUnMHTURI (location)) {
          return;
        }
        
        let nav
        = targetDocument.defaultView
        .QueryInterface (Ci.nsIInterfaceRequestor)
        .getInterface (Ci.nsIWebNavigation);
        let param = UnMHTScheme.getUnMHTURIParam (location);
        
        if (param.scheme != "cache"
            && param.scheme != "cid"
            && param.scheme != "dummy"
            && param.scheme != "empty"
            && param.scheme != "error") {
          let uri;
          try {
            uri = Services.io.newURI (param.original, null, null);
          }
          catch (e) {
            return;
          }
          
          UnMHTFileLoader.loadMHT
            (window,
             uri,
             nav,
             false,         /* ドキュメント無し */
             true,          /* 履歴は置き換える */
             true,          /* 展開する */
             param.refName, /* 現在開いているファイルを開く */
             false);        /* キャッシュは使用しない */
        }
        else {
          nav.loadURI
          (location, nav.LOAD_FLAGS_REPLACE_HISTORY, null, null, null);
        }
      }.bind (this));
    }
  }
};
