/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHT.jsm");
XPCOMUtils.importRelative (this, "UnMHTFileLoader.jsm");
XPCOMUtils.importRelative (this, "UnMHTInfoPanel.jsm");
XPCOMUtils.importRelative (this, "UnMHTKeyboardShortcut.jsm");
XPCOMUtils.importRelative (this, "UnMHTMAFIntegration.jsm");
XPCOMUtils.importRelative (this, "UnMHTMenu.jsm");
XPCOMUtils.importRelative (this, "UnMHTPageInfo.jsm");
XPCOMUtils.importRelative (this, "UnMHTScrapBookIntegration.jsm");
XPCOMUtils.importRelative (this, "UnMHTShutdown.jsm");
XPCOMUtils.importRelative (this, "UnMHTStatusbarProgress.jsm");
XPCOMUtils.importRelative (this, "UnMHTToolbarButton.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowHookLoader.jsm");

let EXPORTED_SYMBOLS = ["UnMHTStartupWindowHook"];

/**
 * ウィンドウへの組み込み
 *
 * @implements IUnMHTStartupListener
 */
let UnMHTStartupWindowHook = {
  /* ==== IUnMHTStartupListener インターフェース ==== */
  
  /**
   * スタートアップ時のイベント
   *
   * @param  Object data
   *         Bootstrap データ
   * @param  Number reason
   *         理由
   */
  onStartup : function (data, reason) {
    let browserHooks = [
      UnMHTMenuClass,
      UnMHTToolbarButtonClass,
      UnMHTInfoPanelClass,
      UnMHTKeyboardShortcut,
      UnMHTFileLoadHook,
      UnMHTScrapBookIntegration,
      UnMHTMAFIntegration,
      UnMHTStatusbarProgress
      ];

    let mailHooks = [
      UnMHTMenuClass,
      UnMHTToolbarButtonClass,
      UnMHTMAFIntegration,
      UnMHTStatusbarProgress
      ];

    let overlays = {
      "chrome://browser/content/browser.xul" : {
        uri : "chrome://unmht/content/ui-browser.xul",
        props : {
          UnMHT: UnMHT
        },
        hooks : browserHooks
      },
      "chrome://navigator/content/navigator.xul" : {
        uri : "chrome://unmht/content/ui-browser.xul",
        props : {
          UnMHT: UnMHT
        },
        hooks : browserHooks
      },
      "chrome://messenger/content/messenger.xul" : {
        uri : "chrome://unmht/content/ui-messenger.xul",
        props : {
          UnMHT: UnMHT
        },
        hooks : mailHooks
      },
      "chrome://browser/content/pageinfo/pageInfo.xul" : {
        uri : "",
        hooks : [
          UnMHTPageInfo
          ]
      }
    };

    let dialogs = [
      "chrome://unmht/content/pref/pref.xul",
      "chrome://unmht/content/additional.xul",
      "chrome://unmht/content/confirm.xul",
      "chrome://unmht/content/nglist.xul",
      "chrome://unmht/content/save.xul",
      "chrome://unmht/content/savemulti.xul",
      "chrome://unmht/content/saveoptions.xul"
      ];
    
    
    let loader = new UnMHTWindowHookLoader (overlays, dialogs);
    loader.onStartup (data, reason);
    UnMHTShutdown.addListener (function (data, reason) {
        loader.onShutdown (data, reason);
        loader.destruct ();
      });
  }
};
