/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHT.jsm");
XPCOMUtils.importRelative (this, "UnMHTPref.jsm");
XPCOMUtils.importRelative (this, "UnMHTPrototype.jsm");
XPCOMUtils.importRelative (this, "UnMHTSave.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowHook.jsm");
XPCOMUtils.importRelative (this, "UnMHTWindowUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTToolbarButtonClass"];

/**
 * ツールバーのボタン管理
 *
 * @param  nsIDOMWindow window
 *         対象のウィンドウ
 * @inherits   UnMHTWindowHook
 * @implements nsISupportsWeakReference
 * @implements nsIWebProgressListener
 * @implements IUnMHTWindowHook
 */
function UnMHTToolbarButtonClass (window) {
  UnMHTWindowHook.call (this, window);
}
UnMHTToolbarButtonClass.prototype = {
  QueryInterface : XPCOMUtils.generateQI ([Ci.nsISupportsWeakReference,
                                           Ci.nsIWebProgressListener]),
  
  /**
   * ツールバーのボタンをクリックしたイベント
   *
   * @param  Event event
   *         対象のイベント
   */
  onClick : function (event) {
    if (("button" in event && event.button != 0)
        || event.ctrlKey || event.shiftKey
        || event.altKey || event.metaKey) {
      return;
    }
    
    if (event.target
        && event.target.nodeName
        && event.target.nodeName.match (/menuitem/i)) {
      return;
    }
    
    UnMHT.showPrefDialog (this._window);
  },
  
  /* ==== nsIWebProgressListener インターフェース ==== */
  
  /**
   * 監視ウィンドウのロケーションが変わった時のイベント
   *   nsIWebProgressListener.onLocationChange
   *
   * @param  nsIWebProgress webProgress
   *         呼び出し元
   * @param  nsIRequest request
   *         状況の変わったリクエスト
   * @param  nsIURI location
   *         ロケーション
   */
  onLocationChange : function (webProgress, request, location) {
    this._setStatus ();
  },
  
  /**
   * 進行状況が変わった時のイベント
   * 未使用
   */
  onProgressChange: function () {
  },
  
  /**
   * プロトコルのセキュリティ設定が変わった時のイベント
   * 未使用
   */
  onSecurityChange : function () {
  },
  
  /**
   * 状況が変わった時のイベント
   * 未使用
   */
  onStateChange : function () {
  },
  
  /* ==== IUnMHTWindowHook インターフェース ==== */
  
  /**
   * 破棄
   */
  destruct : function () {
    UnMHTPrototype.super (this, "destruct").call (this);
  },
  
  /**
   * ウィンドウにロードする
   *
   * @param  Boolean isStartup
   *         スタートアップイベントか
   * @param  Number reason
   *         スタートアップの理由
   */
  load : function (isStartup, reason) {
    this._window.UnMHTToolbarButton = this;
    
    let target = UnMHTWindowUtil.getBrowser (this._window);
    if (target) {
      target.addProgressListener (this);
    }
    
    let f;
    let panel = this.el ("unmht-statusbarpanel-preferences");
    if (panel) {
      f = function () {
        panel.hidden = !UnMHTPref.getPref ("statusbar.icon");
      };
      UnMHTPref.addListener (UnMHTPref.prefix + "statusbar.icon", f, this);
      f ();
    }
    
    this._setStatus ();
  },
  
  /**
   * ウィンドウからアンロードする
   *
   * @param  Boolean isShutdown
   *         シャットダウンイベントか
   * @param  Number reason
   *         シャットダウンの理由
   */
  unload : function (isShutdown, reason) {
    UnMHTPref.removeAllListeners (this);
    
    let target = UnMHTWindowUtil.getBrowser (this._window);
    if (target) {
      target.removeProgressListener (this);
    }
    
    if ("UnMHTToolbarButton" in this._window) {
      delete this._window.UnMHTToolbarButton;
    }
  },
  
  /* ==== private ==== */

  
  /**
   * ツールバーのボタンの状況に合わせて切り替える
   */
  _setStatus : function () {
    let targetDocument = UnMHTWindowUtil.getCurrentDocument (this._window);
    
    let savable
    = UnMHTSave.isSavable (this._window, targetDocument);
    
    let enablePanel = function (type, enabled) {
      let panel = this.el ("unmht-toolbarbutton-" + type);
      if (panel) {
        panel.disabled = !savable;
      }
    }.bind (this);
    
    enablePanel ("save", !savable);
    enablePanel ("quicksave", !savable);
  }
};
UnMHTPrototype.createInheritance (UnMHTToolbarButtonClass, UnMHTWindowHook);
