/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;

let EXPORTED_SYMBOLS = ["UnMHTTreeScanner"]

/**
 * DOM ツリー走査
 *
 * @param  nsIDOMElement startNode
 *         開始要素
 */
function UnMHTTreeScanner (startNode) {
  this._gen = this._getGenerator (startNode);
}
UnMHTTreeScanner.prototype = {
  progress : 0,     /* Number  進捗状況 (0-1) */
  isHead : false,   /* Boolean  HEAD 要素内か */
  
  /**
   * 破棄
   */
  destruct : function () {
    if (this._gen) {
      this._gen.close (); 
      this._gen = null;
    }
  },
  
  /**
   * 次の要素を返す
   *
   * @return nsIDOMNode
   *         次の要素
   */
  getNext : function () {
    return this._gen.next (); 
  },
  
  /* ==== private ==== */
  
  _gen : null,   /* Generator  DOM ツリー操作イテレータ */
  
  /**
   * 要素を返すジェネレータを作成する
   *
   * @param  nsIDOMElement startNode
   *         開始要素
   * @return Generator
   *         要素を返すジェネレータ
   */
  _getGenerator : function (startNode) {
    let node = startNode;
    this.isHead = false;
    
    let i = 0;
    let nodeCount = 1;
    try {
      nodeCount = startNode.getElementsByTagName ("*").length + 1;
    }
    catch (e) {
    }
    while (node) {
      if (node instanceof Ci.nsIDOMHTMLHeadElement) {
        this.isHead = true;
      }
      
      this.progress = i / nodeCount;
      
      yield node;
      if (node instanceof Ci.nsIDOMElement) {
        i ++;
      }
      
      if (node.firstChild) {
        node = node.firstChild;
      }
      else {
        while (node) {
          if (node.nextSibling) {
            if (node instanceof Ci.nsIDOMHTMLHeadElement) {
              this.isHead = false;
            }
            
            node = node.nextSibling;
            break;
          }
          node = node.parentNode;
          if (node == startNode) {
            this.progress = 1;
            yield false;
            return;
          }
        }
      }
    }
  }
};
