/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;

let EXPORTED_SYMBOLS = ["UnMHTUnicodeConverter"];

/**
 * 文字列変換器
 */
let UnMHTUnicodeConverter = {
  /**
   * Unicode から変換する
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換先の文字コード
   * @return String
   *         変換した文字列
   */
  convertFromUnicode : function (text, charset) {
    try {
      let converter
      = Cc ["@mozilla.org/intl/scriptableunicodeconverter"]
      .createInstance (Ci.nsIScriptableUnicodeConverter);
      converter.charset = charset;
      return converter.ConvertFromUnicode (text) + converter.Finish ();
    }
    catch (e) {
    }
            
    return text;
  },
  
  /**
   * Unicode から変換する
   * 変換できないものは実体参照に置き換える
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換先の文字コード
   * @return String
   *         変換した文字列
   */
  convertFromUnicodeWithEntity : function (text, charset) {
    let text2
    = this.convertToUnicode (this.convertFromUnicode (text, charset), charset);
    
    let result = "";
    for (;;) {
      let p = text2.indexOf ("?");
      if (p == -1) {
        result += text2;
        break;
      }
      
      if (text.charAt (p) != "?") {
        /* 変換できなかったため ? になった */
        
        result += text2.substr (0, p);
        
        let code1 = text.charCodeAt (p);
        if (code1 >= 0xd800 && code1 <= 0xdbff) {
          /* サロゲートペア */
          let code2 = text.charCodeAt (p + 1);
          let code = (code1 - 0xd800) * 0x400 + (code2 - 0xdc00) + 0x10000;
          
          result += "&#" + code + ";";
          
          text = text.substr (p + 2);
          text2 = text2.substr (p + 1);
        }
        else {
          result += "&#" + code1 + ";";
          text = text.substr (p + 1);
          text2 = text2.substr (p + 1);
        }
      }
      else {
        /* 本当に ? だった */
        result += text2.substr (0, p + 1);
        text = text.substr (p + 1);
        text2 = text2.substr (p + 1);
      }
    }
    
    return this.convertFromUnicode (result, charset);
  },
  
  /**
   * Unicode に変換する
   *
   * @param  String text
   *         変換する文字列
   * @param  String charset
   *         変換元の文字コード
   * @return String
   *         変換した文字列
   */
  convertToUnicode : function (text, charset) {
    try {
      let converter
      = Cc ["@mozilla.org/intl/scriptableunicodeconverter"]
      .createInstance (Ci.nsIScriptableUnicodeConverter);
      converter.charset = charset;
      return converter.ConvertToUnicode (text);
    }
    catch (e) {
    }
            
    return text;
  }
};
