/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
"use strict";

let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;
let Cu = Components.utils;
Cu.import ("resource://gre/modules/XPCOMUtils.jsm");
XPCOMUtils.importRelative (this, "UnMHTArrayUtil.jsm");

let EXPORTED_SYMBOLS = ["UnMHTWinRegUtil"];

/**
 * Windows レジストリ操作ユーティリティ
 */
let UnMHTWinRegUtil = {
  /**
   * HKEY_LOCAL_MACHINE 内の値を取得する
   *
   * @param  Array paths
   *         レジストリのパス
   *           [String, ...]
   * @param  String name
   *         パス内の名前
   * @return String/Number
   *         値
   */
  getHKLMValue : function (paths, name) {
    return this._getValue
    (Ci.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE, paths, name);
  },
  
  /**
   * HKEY_CURRENT_USER 内の値を取得する
   *
   * @param  Array paths
   *         レジストリのパス
   *           [String, ...]
   * @param  String name
   *         パス内の名前
   * @return String/Number
   *         値
   */
  getHKCUValue : function (paths, name) {
    return this._getValue
    (Ci.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER, paths, name);
  },
  
  /**
   * HKEY_CLASSES_ROOT 内の値を取得する
   *
   * @param  Array paths
   *         レジストリのパス
   *           [String, ...]
   * @param  String name
   *         パス内の名前
   * @return String/Number
   *         値
   */
  getHKCRValue : function (paths, name) {
    return this._getValue
    (Ci.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE, paths, name);
  },
  
  /**
   * 指定されたルートキー内の値を取得する
   *
   * @param  Number rootKey
   *         ルートキー
   *           Ci.nsIWindowsRegKey.ROOT_KEY_*
   * @param  Array paths
   *         レジストリのパス
   *           [String, ...]
   * @param  String name
   *         パス内の名前
   * @return String/Number
   *         値
   */
  _getValue : function (rootKey, paths, name) {
    let regKey
    = Cc ["@mozilla.org/windows-registry-key;1"]
    .createInstance (Ci.nsIWindowsRegKey);
    regKey.open (rootKey, paths [0], Ci.nsIWindowsRegKey.ACCESS_READ)
    
    let subRegKeys
    = UnMHTArrayUtil
    .scanl (paths.slice (1),
            (function (regKey, path)
             regKey
             ? (regKey.hasChild (path)
                ? regKey.openChild (path, Ci.nsIWindowsRegKey.ACCESS_READ)
                : null)
             : null),
            regKey);
    
    let targetKey = subRegKeys [subRegKeys.length - 1];
    
    let value = null;
    if (targetKey) {
      switch (targetKey.getValueType (name)) {
        case Ci.nsIWindowsRegKey.TYPE_STRING: {
          value = targetKey.readStringValue (name);
          break;
        }
        case Ci.nsIWindowsRegKey.TYPE_BINARY: {
          value = targetKey.readBinaryValue (name);
          break;
        }
        case Ci.nsIWindowsRegKey.TYPE_INT: {
          value = targetKey.readIntValue (name);
          break;
        }
        case Ci.nsIWindowsRegKey.TYPE_INT64: {
          value = targetKey.readInt64Value (name);
          break;
        }
      }
    }
    
    subRegKeys.forEach (function (key) (key && key.close ()));
    regKey.close ();
    
    return value;
  }
};
